/* 
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/basis.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 basis() establishes the basis function. The basis can be either "blob" or "voronoi". If blob basis, then one should also specify the support of the blob "blob.support", shape of the blob "Blob.shape", and the grid resolution "Blob.delta"

 */

#include <cstdio>
#include <cmath>
#include <cstdlib>

#include "consts.h"
#include "blkdta.h"
#include "blob.h"
#include "geom.h"
#include "basis.h"
#include "uiod.h"
#include "int2str.h"
#include "infile.h"

void basis()
{
	INTEGER word;
	REAL aux;

	BOOLEAN eol;

	static const INTEGER blob = CHAR2INT('b', 'l', 'o', 'b');
	static const INTEGER pixel = CHAR2INT('p', 'i', 'x', 'e');

	// modified calls to getwrd() to use the 4-parameter version. Lajos, Dec 13, 2004
	static const INTEGER basis_codes[2] =
	{ CHAR2INT('b', 'l', 'o', 'b'), CHAR2INT('p', 'i', 'x', 'e') };

	Blob.pix_basis = TRUE;

	for (;;)
	{

		word = InFile.getwrd(FALSE, &eol, basis_codes, 2);

		if (eol)
		{
			break;
		}

		if (word == blob)
		{
			Blob.pix_basis = FALSE;
			//default values
			Blob.delta = 2.0 * Consts.isqrt3 * GeoPar.pixsiz;
			//Blob.delta =   4.0 * Consts.isqrt3 * GeoPar.pixsiz;
			Blob.shape = 11.2828631556;
			Blob.support = 1.7865601396 * Blob.delta;
			//Blob.support = 2 * 1.7865601396 * Blob.delta;
			aux = InFile.getnum(FALSE, &eol);
			if (!eol)
			{
				Blob.support = aux;
				Blob.shape = InFile.getnum(FALSE, &eol);
				if (Blob.shape < 0.0)
				{
					fprintf(output,
							"\n **** shape of a blob cannot be negative");
					fprintf(output, "\n **** program aborted\n");
					exit(333);
				}

				//delta = getnum(NULL, &eol);
				Blob.delta = InFile.getnum(FALSE, &eol);
				if ((Blob.delta <= Consts.zero)
						|| (Blob.support <= Consts.zero))
				{
					fprintf(output,
							"\n **** delta and support must be greater than ZERO");
					fprintf(output, "\n **** program aborted\n");
					exit(333);
				}
			}

			Blob.setparam();
			Blob.table();

			break;
		}

		else if (word == pixel)
		{
			Blob.pix_basis = TRUE;
			break;
		}

		else
		{
			fprintf(output, "\n **** unknown basis %s", int2str(word));
			fprintf(output, "\n **** program aborted\n");
			exit(333);
		}
	}
	// ECHO OPTION SELECTED

	if (Blob.pix_basis)
	{
		fprintf(output, "\n          pixel basis");
	}

	else
	{
		fprintf(output,
				"\n          blob basis with parameters: support = %10.4f shape = %10.4f delta = %10.4f",
				Blob.support, Blob.shape, Blob.delta);

	}
}
