/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/cin.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 THIS FUNCTION CALCULATE THE INTEGRAL OF (1.-COS(U))/U W.R.T. U
 FROM 0 TO X USING RATIONAL APPROXIMATIONS
 REFERENCES :
 1) POLYNOMIAL APPROXIMATIONS TO INTEGRAL TRANSFORMS,
 J. WIMP
 MATHEMATICS OF COMPUTATION, 15, 1961, P.P.177
 2) APPROXIMATIONS FOR DIGITAL COMPUTERS,
 C. HASTINGS
 PRINCETON UNIVERSITY PRESS, 1955, P.P.198
 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"

#include "cin.h"

REAL cin(REAL x)
{

	static const REAL gamma = (REAL) 0.5772156649;
	static const REAL a2n0 = (REAL) 0.1352962627;
	REAL ang;
	REAL anginc;
	static const REAL a2n[6] = { (REAL) -0.4232751922, (REAL) 0.0182227219, (REAL) 0.000415765, (REAL) 56716.e-10, (REAL) -511.e-10, (REAL) 3.e-10 };

	REAL xsq;
	REAL f;
	REAL g;

	if (x == 0)
		return 0;

	x = (REAL) fabs(x);

	if (x < 1)
	{

		REAL cin_tmp = gamma - a2n0;
		ang = 0.;
		anginc = (REAL) 2.0 * (REAL) atan(sqrt(4. - x * x) / x);
		for (int i = 0; i < 6; i++)
		{
			ang += anginc;
			cin_tmp -= a2n[i] * (REAL) cos(ang);
		}
		return cin_tmp;
	}

	xsq = x * x;

	f = ((xsq + (REAL) 7.241163) * xsq + (REAL) 2.463936) / ((xsq + (REAL) 9.068580) * xsq + (REAL) 7.157433) / x;

	g = ((xsq + (REAL) 7.547478) * xsq + (REAL) 1.564072) / ((xsq + (REAL) 12.723684) * xsq + (REAL) 15.723606) / xsq;

	return gamma + (REAL) log(x) - f * (REAL) sin(x) + g * (REAL) cos(x);
}
