/*
**********************************************************************
$SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
$HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/create_phantom.c $
$LastChangedRevision: 85 $
$Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
$Author: agulati $
**********************************************************************

 Previously part of SuperSNARK
 */

/*--------------------------------------------------------------------------
 * Super Snark V 1.0
 * Written by Jingsheng Zheng... November 1993
 * Modified by Jolyon Browne ... June 1993
 * --------------------------------------------------------------------------*/

#include <string.h>
#include <strings.h>
#include <stdlib.h>

#include "experimenter.h"
#include "blanks.h"
#include "errorc.h"
#include "create_phantom.h"

/*--------------------------------------------------------------------------
 * This fuction generates the input lines that defines the elemental
 * objects as required by the CREATE command.
 *
 * INPUTS:
 * phan_flname - phantom filename (Note: the list of phantom filenames are
 * stored in the ensemble data file).
 * seed - seed used for the random number generator.
 * navel, nelem,pixsz - modifiers described by Set 4 of the SNARK CREATE
 * input sequence (see the SNARK manual for details).
 *
 * OUTPUTS:
 * structure - string array of length num_str containing the elemental object
 * description.
 * num_str - number of structures in the phantom.
 * num_pairs - for PAIRed structures, this is the number of pairs of
 * structures.
 * pairs - array of length 2*num_pairs containing the indicies of all PAIRed
 * structures.
 * ----------------------------------------------------------------------------*/

void create_phantom(
        char* phan_flname,
        long seed,
        int navel,
        int nelem,
        float pixsz,
        char*** structure, //jk 01/01/08 modified to remove
        // restriction for number of structures
        int* num_str,
        int* num_pairs,
        int** pairs, //jk 01/01/08 modified to remove
        // restriction for number of structures
        int iroi_used //jk  09/07/2008   added a check to verify that
        //density of one out of each paired  structures is zero
        )
{
    float cx, cy, u, v, ang, den1, den2, prob, tmp;
    float scale, sd;
    int i, nstr = 0, npair = 0, tot_nstr = 0; //jk 01/01/2008 added tot_nstr
    //variable to keep track of the number
    //of structures in the phantom
    int scaleCount = 0; //keeps track of how many times line starting with SCALE
                        //appeared in the input file (it should be exactly 1
                        //at the end of the file construction).

    char shape[10], keyword[4], string[MAXLINESIZE];
    FILE *phanfl;

    *num_pairs = 0;

    //jk 09/20/2008
    char spect[10], spectType[15];
    int erg, nerg;
    int poly = 0;
    int change = 0;
    char toc[14][MAXLINESIZE];
    int polyErg[7];
    int polyProb[7];
    char string_alt[MAXLINESIZE];
    long phantom_seed = (long) (100000000 * drand48() );

    if ((phanfl = fopen(phan_flname, "r")) == NULL) {
        errorc("In create_phantom.c: error in opening file ", phan_flname);
    }

    /* print out beginning of CREATE input sequence */

    fprintf(pstream, "CREATE\n");
    fprintf(pstream, "%s, seed used in experimenter to generate this file = %ld\n", phan_flname, seed);

    // jk 09/20/2008 adding polychromatic data option
    if (fgets(string, sizeof (string), phanfl) == NULL) {
        errorc("error: premature end of file encountered ", phan_flname);
    }
    while ( isSkip(string) ) //it's a comment line,
    {                    	//echo it to output and read next line
        if( isComment(string) )
        {
            fprintf(pstream, "%s", string);
            if ( fgets(string, sizeof (string), phanfl) == NULL) break;
        }
    }
    if (strncasecmp(string, "SPEC", 4) == 0) 
	{
        sscanf(string, "%s%s%i", spect, spectType, &erg);
        
		if (strncasecmp(spectType, "mono", 4) == 0)
            fprintf(pstream, "SPECTRUM MONOCHROMATIC %i\n", erg);
        
		else if (strncasecmp(spectType, "poly", 4) == 0) 
		{
            nerg = erg;
            if (nerg > 7) 
			{
                errorc("error: too many energy levels specified ", phan_flname);
            }
            poly = 1;
            fprintf(pstream, "SPECTRUM POLYCHROMATIC %i\n", nerg);
        }
    }
    if (poly) 
	{
        if (fgets(string, sizeof (string), phanfl) == NULL) 
		{
            errorc("error: premature end of file encountered ", phan_flname);
        }
        while ( isBlank(string) ) //skip blank lines
            if ( fgets(string, sizeof (string), phanfl) == NULL) break;
		
		fprintf(pstream, "%s", string);
    }

    fprintf(pstream, "OBJECTS\n");


    /* jk 01/01/08
     * scan the file to count the structures, then
     * allocate two arrays: structure[][] and pairs[] to hold the info about
     * the structures
     * this allows for "unlimited" number of structures in the phantom
     */
    while ((fgets(string, sizeof (string), phanfl)) != NULL) 
	{

        if (	strncasecmp(string, "dumm", 4) == 0 ||
                strncasecmp(string, "sing", 4) == 0 ||
                strncasecmp(string, "pair", 4) == 0 ||
                strncasecmp(string, "scal", 4) == 0) 
		{
            
			/* keyword is 'dummy','pair', 'single' or 'scale' */
            strncpy(keyword, string, 4);
        } 
		
		else if ( isSkip(string) ) //the line is a comment or blank, skip it
		{;} 
		
		else if (strncasecmp(keyword, "pair", 4) == 0) 
		{
            tot_nstr = tot_nstr + 2;
        } 
		
		else if (strncasecmp(keyword, "sing", 4) == 0) 
		{
            tot_nstr = tot_nstr + 1;
        }
        /*else do nothing - DUMMY structures do not count */
    }

    //allocate the arrays
    *pairs = calloc(tot_nstr, sizeof (int));

    *structure = calloc(tot_nstr, sizeof (char*));

    for (i = 0; i < tot_nstr; i++) 
	{
        (*structure)[i] = calloc(MAXLINESIZE, sizeof (char));
    }

    //set pointer to the file at its beginning
    rewind(phanfl);

    while ((fgets(string, sizeof (string), phanfl)) != NULL) 
	{

        if (	strncasecmp(string, "dumm", 4) == 0 ||
                strncasecmp(string, "sing", 4) == 0 ||
                strncasecmp(string, "pair", 4) == 0 ||
                strncasecmp(string, "scal", 4) == 0 ||
                strncasecmp(string, "spec", 4) == 0) 
		{
            /* keyword is 'dummy','pair', */
            strncpy(keyword, string, 4); /* 'single' or 'scale'  or 'spec'*/
        }
		
		else if ( isComment(string) ) //the line is a comment, echo it to pstream
		{
			fprintf(pstream, "%s", string);
		}
		
		else if ( isBlank(string) ) //the line is blank, skip it
		{;}
		
        else if (strncasecmp(keyword, "spec", 4) == 0) 
		{
            //ignore the next line after spec, if any; if was written to pstream before
        } 
		
		else if (strncasecmp(keyword, "pair", 4) == 0) 
		{
			 /* if a paired structure:
			 (a) increment number of pairs read in;
			 (b) read in description of pair;
			 (c) errorchk2: make sure paired objects have unequal densities;
			 (d) randomly assign densities and save first structure;
			 (e) print out structure description onto SNARK input stream;
			 (f) save second structure and also print this structure's
			 description onto SNARK input stream. 
			 */

            if (strlen(string) - 1 != blanks(string) && isValidObject(string) ) 	/* non-blank line */
            {   
				(*num_pairs)++;
                sscanf(string, "%s%f%f%f%f%f%f%f%f", shape, &cx, &cy, &u, &v,
                        &ang, &den1, &den2, &prob);
                errorchk2(den1, den2, prob, iroi_used);
                if (poly)  //read the next two lines with polychromatic densities
                {
					fgets(string, sizeof (string), phanfl);
                    while ( isSkip(string) ) //it's a comment line,
                    {                    //echo it to output and read next line
                        if( isComment(string) )     fprintf(pstream, "%s", string);
                        if ( fgets(string, sizeof (string), phanfl) == NULL) break;
                    }

                    fgets(string_alt, sizeof (string), phanfl);
                    while ( isSkip(string) ) //it's a comment line,
                    {                    //echo it to output and read next line
                        if( isComment(string) )     fprintf(pstream, "%s", string);
                        if ( fgets(string, sizeof (string), phanfl) == NULL) break;
                    }
                }
                change = 0;
                if (drand48() > prob) 
				{
                    tmp = den1;
                    den1 = den2;
                    den2 = tmp;
                    change = 1;
                }
                sprintf((*structure)[nstr], "%s %f %f %f %f %f", shape, cx, cy, u, v, ang);
                (*pairs)[npair++] = nstr + 1;
                fprintf(pstream, "%s %f\n", (*structure)[nstr++], den1);
                if (poly)  //print the remaining densities for polychromatic
                {    
					if (change) fprintf(pstream, "%s", string_alt);
                    else fprintf(pstream, "%s", string);
                }
                sprintf((*structure)[nstr], "%s %f %f %f %f %f", shape, -cx, cy, u, v, -ang);
                (*pairs)[npair++] = nstr + 1;
                fprintf(pstream, "%s %f\n", (*structure)[nstr++], den2);
                if (poly)  //print the remaining densities for polychromatic
                {
					if (change) fprintf(pstream, "%s", string);
                    else fprintf(pstream, "%s", string_alt);
                }
            }  
			else errorc("Wrong command string in ", phan_flname);
        } 
		else if (strncasecmp(keyword, "sing", 4) == 0) 
		{
			/* if a single structure then print out its description onto
			SNARK's input stream. Also save this structure's description */

            if (strlen(string) - 1 != blanks(string) && isValidObject(string) ) 	 /* non-blank line */
            {
				fprintf(pstream, "%s", string);
                sscanf(string, "%s%f%f%f%f%f", shape, &cx, &cy, &u, &v, &ang);
                sprintf((*structure)[nstr++], "%s %f %f %f %f %f", shape, cx, cy, u, v, ang);
                if (poly)  //read and print the next  line with polychromatic densities
                {
					fgets(string, sizeof (string), phanfl);
                    while ( isSkip(string) ) //it's a comment line,
                    {                    //echo it to output and read next line
                        if( isComment(string) )     fprintf(pstream, "%s", string);
                        if ( fgets(string, sizeof (string), phanfl) == NULL) break;
                    }
                    fprintf(pstream, "%s", string);
                }
            }  
			else errorc("Wrong command string in ", phan_flname);
        }/* for 'scale' and 'dummy' merely echo the input */
		
		else if (strncasecmp(keyword, "scal", 4) == 0) 
		{
			tmp = sscanf(string, "%f%f", &scale, &sd);
            if (tmp == 2)   //print all: scale, phantom_seed and sd
            {
				if (sd <= 0)  errorc("Value of standard deviation cannot be negative "
                                           "in SCALE followup line in ", phan_flname);
                fprintf(pstream, "last %f %ld %f \n", scale, phantom_seed, sd);
            }
            else if (tmp == 1)   //print only scale
                fprintf(pstream, "last %f \n", scale);
            else
                errorc("Invalid SCALE followup line in ", phan_flname);
            scaleCount++;
        } 
		else if (strncasecmp(keyword, "dumm", 4) == 0) 
		{
            if (strlen(string) - 1 != blanks(string) && isValidObject(string) ) 
			{
                fprintf(pstream, "%s", string);
                if (poly) { //read and print the next line with polychromatic densities
                    fgets(string, sizeof (string), phanfl);
                    while ( isSkip(string) ) //it's a comment line,
                    {                    //echo it to output and read next line
                        if( isComment(string) )     fprintf(pstream, "%s", string);
                        if ( fgets(string, sizeof (string), phanfl) == NULL) break;
                    }
                    fprintf(pstream, "%s", string);
                }
            }  
			else errorc("Wrong command string in ", phan_flname);
			
        } 
		else errorc("Wrong command string in ", phan_flname);
		
    }

    if( scaleCount != 1)
        errorc("SCALE and its followup line should appear exactly once in ", phan_flname);

    *num_str = nstr;
    fclose(phanfl);
    fprintf(pstream, "phantom average %d\n", navel);
    fprintf(pstream, "%d %f \n", nelem, pixsz);
}

/*
 * This function checks to make sure that PAIRed structures do
 * not have the same density. Also checks to ensure that the probability
 * flag is a valid number
 */

void errorchk2(float den1, float den2, float prob, int iroi_used)
{
    if (den1 == den2) 
	{
        errorc("error: ", "paired structures with equal densities");
    }

    if (prob > 1 || prob < 0) 
	{
        errorc("error: ", "invalid probability for paired structures");
    }

    //jk 09/07/2008
    //added a check to verify that density of one out of each 
    //paired  structures is zero
    if (iroi_used) 
	{
        if (den1 != 0) 
		{
            if (den2 != 0) 
			{
                errorc("error: ", "when IROI is used, one of the paired structures "
                        "has to have density of zero, the other non-zero");
            } 
        } 
		else 
		{
            if (den2 != 0) 
			{ 
			//that's fine
            } 
			else 
			{
                errorc("error: ", "when IROI is used, one of the paired structures "
                        "has to have density of zero, the other non-zero");
            }

        }

    }

}
