/*
**********************************************************************
$SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
$HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/dcon_dconft.cpp $
$LastChangedRevision: 85 $
$Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
$Author: agulati $
**********************************************************************

 THIS FUNCTION IS USED IN DIVERGENT BEAM CONVOLUTION ALGORITHM
 TO CALCULATE THE CONVOLUTING FUNCTIONS Q1(M) AND Q2(M) FOR
 ARC DETECTOR CASE
*/

#include <cmath>
#include <cstdio>

#include "blkdta.h"
#include "region.h"
#include "consts.h"
#include "uiod.h"
#include "int2str.h"
#include "infile.h"
#include "cin.h"

#include "dcon.h"

BOOLEAN dcon_class::dconft(REAL* q1, REAL* q2, INTEGER nconrp, REAL alfa, INTEGER nf)
{
  
  INTEGER m;
  REAL aa;
  REAL ya;
  REAL bb;
  REAL yb;
  REAL a;
  REAL b;
  REAL slope;
  REAL sigma;
  REAL api;
  REAL bpi;
  REAL ampi;
  REAL bmpi;
  REAL sinsgm;
  REAL sinamp;
  REAL cosamp;
  REAL sinbmp;
  REAL cosbmp;
  REAL c;
  REAL cpi;
  REAL cm;
  REAL cmp;
  REAL alpha;
  INTEGER is;
  REAL scmp;
  REAL ccmp;
  REAL cm2;
  REAL cmp2;
  REAL e;
  REAL xcin;
  REAL cinf;

  REAL y0;
  REAL p;
  REAL cr;

  
  BOOLEAN eol;

  static const INTEGER hline = CHAR2INT('l','i','n','e');
  static const INTEGER hhamm = CHAR2INT('h','a','m','m');
  static const INTEGER hpara = CHAR2INT('p','a','r','a');
  static const INTEGER hexpo = CHAR2INT('e','x','p','o');
  static const INTEGER hsinc = CHAR2INT('s','i','n','c');
  static const INTEGER hcosi = CHAR2INT('c','o','s','i');
  static const INTEGER hshep = CHAR2INT('s','h','e','p');


  // INITIALIZE CONSTANTS FOR CONVOLUTING FUNCTION

  for(m = 0; m < nconrp; m++) 
  {
    q1[m] = 0.;
    q2[m] = 0.;
  }

  // FILTER FUNCTION DEFINED BY LINES WHICH CONNECTING THE COORDINATES
  // OF (AA,YA) AND (BB,YB)

  if(nf != hline) goto L50;
  fprintf(output, "\n          filter function composed of following lines");

  aa = 0.;
  ya = 1.;
  bb = InFile.getnum(TRUE, &eol);

  if(eol) goto L400;
  yb = InFile.getnum(FALSE, &eol);

  if(eol) yb = InFile.getnum(TRUE, &eol);
  if(eol) goto L400;

L20:
  if((aa-bb) > 0) 
  { 
L25:
    fprintf(output, "\n          unrealistic parameters aa = %10.4f   bb = %10.4f", aa, bb);
    fprintf(output, "\n                                 ya = %10.4f   yb = %10.4f", ya, yb);
	return TRUE;
  }

  if((aa-bb) < 0) 
  { 
    if((ya < yb) || (bb > 1) || (yb < 0)) goto L25;

    a = aa / (REAL) 2.0 / alfa;
    b = bb / (REAL) 2.0 / alfa;
    slope = (yb - ya) / (b - a);
    y0 = ya - a * slope;
    sigma = alfa;
    api = aa * Consts.pi;
    bpi = bb * Consts.pi;
    ampi = api;
    bmpi = bpi;
    fprintf(output, "\n            (%6.4f,%6.4f) to (%6.4f,%6.4f)", aa, ya, bb, yb);

    p = (REAL) -4.0 * Consts.pisq * (y0*(b+a)*(b-a) / (REAL) 2.0 + slope * (b*b*b-a*a*a) / (REAL) 3.0);
    q1[0] = q1[0] + p;                           //(JD 11/06/03)
    q2[0] = q2[0] - (REAL) 2.0 * p;              //(JD 11/06/03)

    for(m = 1; m < nconrp; m++) 
	{
		sinsgm = (REAL) sin(sigma);
		sinamp = (REAL) sin(ampi);
		cosamp = (REAL) cos(ampi);
		sinbmp = (REAL) sin(bmpi);
		cosbmp = (REAL) cos(bmpi);

		q1[m] -= (ya * cosamp - yb * cosbmp + slope * (sinbmp - sinamp) / Consts.twopi / sigma) / sinsgm / sinsgm;
		q2[m] += (Consts.twopi * (yb * b * sinbmp - ya * a * sinamp) + slope * (b * cosbmp - a * cosamp - (sinbmp - sinamp) / Consts.twopi / sigma) / sigma) / sinsgm;

		sigma += alfa;
		ampi += api;
		bmpi += bpi;
    }
  }

  if((yb != 0.)) 
  {
    aa = bb;
    ya = yb;
    bb = InFile.getnum(FALSE, &eol);

    if(eol) bb = InFile.getnum(TRUE, &eol);
    if(eol) goto L400;

    yb = InFile.getnum(FALSE, &eol);

    if(eol) yb = InFile.getnum(TRUE, &eol);

    if(eol) goto L400;
    goto L20;
  }
  return FALSE;

  // FILTER BANDWIDTH -  A=C/ALFA

L50:
  c = InFile.getnum(TRUE, &eol);
  if(eol) goto L400;
  if((c <= 0.) || (c > 1.)) c = 1.;
  cpi = c * Consts.pi;
  sigma = alfa;
  cm = c;
  cmp = cpi;

//.... HAMMING LOW PASS FILTER
//....   F(X)=ALPHA+(1.-ALPHA)*COS(2.*PI*X/A)

  if(nf != hhamm) goto L90;
  alpha = InFile.getnum(FALSE, &eol);

  if(eol) alpha = 1.;
  fprintf(output, "\n          hamming low pass filter");
  fprintf(output, "\n           alpha= %7.3f", alpha);

  cr = (REAL) 1.0 / c;
  Region.d1 = (REAL) 1.0 - alpha;
  q1[0] = (REAL) 0.5 * SQR(c / alfa) * ((REAL) 4.0 * Region.d1 - alpha * Consts.pisq);  //(JD 11/06/03)
  q2[0] = (REAL) -2.0 * q1[0];                //(JD 11/06/03)
  is = 1;                                     //(JD 11/06/03)

L60:
	for(m = is; m < nconrp; m++) 			//(JD 11/06/03)
	{          
		if(fabs(((float)(m)) - cr) < Consts.zero) goto L80;          //(JD 11/06/03)
		scmp = (REAL) sin(cmp);
		ccmp = (REAL) cos(cmp);
		b = (REAL) sin(sigma);
		cm2 = cm * cm;
		q1[m] = -(cm2 - alpha + ((Region.d1 - alpha) * cm2 + alpha) * ccmp) / b / b / (cm2 - (REAL) 1.0);
		q2[m] = c * ((cm2 - (REAL) 1.0) * (cm2 * (alpha - Region.d1) - alpha) * Consts.pi * scmp - (REAL) 2.0 * cm * Region.d1 * ((REAL) 1.0 + ccmp)) / b / (REAL) SQR(cm2 - (REAL) 1.0) / alfa;
		sigma += alfa;
		cmp += cpi;
		cm += c;
	}
  goto L600;

L80:
  b = (REAL) sin(sigma);
  q1[m] = (REAL) -2.0 * alpha / b / b;
  q2[m] = (REAL) 0.25 * Region.d1 * c * Consts.pisq / b / alfa;
  is = m+1;
  sigma += alfa;
  cm += c;
  cmp += cpi;
  goto L60;


L90:
  if(nf != hpara) goto L110;
  fprintf(output, "\n           parabolic low pass filter");

  q1[0] = (REAL) -0.25 * SQR(cpi/alfa);           //(JD 11/06/03)
  q2[0] = (REAL) -2.0 * q1[0];                    //(JD 11/06/03)

  for(m = 1; m < nconrp; m++) {
    scmp = (REAL) sin(cmp);
    ccmp = (REAL) cos(cmp);
    b = (REAL) sin(sigma);
    cmp2 = cmp * cmp;
    q1[m] = ((REAL) 2.0 *(cmp * scmp - (REAL) 1.0 + ccmp) - cmp2) / (REAL) SQR(b*cmp);
    q2[m] = (REAL) 2.0 *((REAL) 2.0*cmp*scmp+((REAL) 2.0-cmp2)*ccmp-(REAL) 2.0) / b / sigma / cmp2;
    sigma += alfa;
    cmp += cpi;
  }
  goto L600;


L110:
	if(nf != hexpo) goto L130;
	fprintf(output, "\n           exponential low pass filter");

	e = (REAL) 2.7182818;
	q1[0] = (REAL) -0.5 * (e - (REAL) 2.0) / (e - (REAL) 1.0) * SQR(cpi/alfa);       //(JD 11/06/03)
	q2[0] = (REAL) -2. * q1[0];                                                      //(JD 11/06/03)
	for(m = 1; m < nconrp; m++) 
	{
		scmp = (REAL) sin(cmp);
		ccmp = (REAL) cos(cmp);
		b = (REAL) sin(sigma);
		cmp2 = cmp * cmp;
		q1[m] = -e / (e - (REAL) 1.0) * ((REAL) 1.0 - ccmp + cmp * ((e - (REAL) 1.0) * cmp / e - scmp)) / b / b / ((REAL) 1.0 + cmp2);
		q2[m] = e / (e - (REAL) 1.0) * cmp2 * ((REAL) -2.0 / e + ((REAL) 1.0 - cmp2) * ccmp + (REAL) 2.0 * cmp * scmp) / b / sigma / SQR((REAL) 1.0 + cmp2);
		sigma += alfa;
		cmp += cpi;
	}
  goto L600;


L130:
  if(nf != hsinc) goto L160;
  fprintf(output, "\n           sinc low pass filter");

  cr = (REAL) 1.0 / c;
  q1[0] = (REAL) -2.0 * SQR(c/alfa);                  //(JD 11/06/03)
  q2[0] = (REAL) -2.0 * q1[0];                        //(JD 11/06/03)
  is = 1;                                             //(JD 11/06/03)

L140:
	for(m = is; m < nconrp; m++) 
	{                  //(JD 11/06/03)
		b = (REAL) sin(sigma);
		xcin = cin(cmp - Consts.pi) - cin(cmp + Consts.pi);
		q1[m] = (REAL) 0.5 * cm * xcin / b / b;
		if(fabs(((float)(m)) - cr) < Consts.zero) goto L155;       //(JD 11/19/03)
		q2[m] = c * ((REAL) -0.5 * xcin - cm * ((REAL) 1.0 + (REAL) cos(cmp)) / (cm * cm - (REAL) 1.0)) / b / alfa;
		cm += c;
		sigma += alfa;
		cmp += cpi;
	}
  goto L600;

L155:
  q2[m] = (REAL) 0.5 * c * cin(Consts.pi + Consts.pi) / b / alfa;
  cm += c;
  sigma += alfa;
  cmp += cpi;
  is = m+1;
  goto L140;


L160:
  if(nf != hcosi) goto L200;
  fprintf(output, "\n           cosine low pass filter");

  cr = (REAL) 0.5 / c;
  q1[0] = (REAL) 2.0 * ((REAL) 2.0 - Consts.pi) * SQR(c / alfa);     //(JD 11/06/03)
  q2[0] = (REAL) -2.0 * q1[0];                                       //(JD 11/06/03)
  is=1;                                                              //(JD 11/06/03)

L170:


	for(m=is; m < nconrp; m++) 										//(JD 11/06/03)
	{                                      
		b = (REAL) sin(sigma);
		if(fabs(float(m) - cr) < Consts.zero) goto L190;     //(JD 11/19/03)
		scmp = (REAL) sin(cmp);
		ccmp = (REAL) cos(cmp);
		cm2 = (REAL) 4.0 * cm * cm;
		q1[m] = ((REAL) 2.0 * cm * scmp - cm2) / b / b / (cm2 - (REAL) 1.0);
		q2[m] = (REAL) 2.0 * c / alfa * ((REAL) -4.0 * cm + (cm2 + (REAL) 1.0) * scmp - cmp * (cm2 - (REAL) 1.0) * ccmp) / SQR(cm2 - (REAL) 1.0) / b;
		cm += c;
		sigma += alfa;
		cmp += cpi;
	}

  goto L600;

L190:
  q1[m] = (REAL) -0.5 / b / b;
  q2[m] = (REAL) 0.125 * ((REAL) 4.0 + Consts.pisq) * c / alfa / b;
  cm += c;
  sigma += alfa;
  cmp += cpi;
  is = m + 1;
  goto L170;

  // SHEPP AND LOGAN'S SINC FILTER

L200:
  if(nf != hshep) goto L240;
  fprintf(output, "\n           shepp and logan's sinc filter");

  q1[0] = (REAL) -4.0 * SQR(c/alfa);                //(JD 11/06/03)
  q2[0] = (REAL) -2.0 * q1[0];                      //(JD 11/06/03)
  cr = (REAL) 0.5 / c;
  is = 1;                                           //(JD 11/06/03)

L210:

	for(m = is; m < nconrp; m++) 					 //(JD 11/06/03)
	{                          
		b = (REAL) sin(sigma);
		cm2 = cm * cm;
		cinf = cin(cmp + Consts.pid2) - cin(cmp - Consts.pid2);
		q1[m] = -cm * cinf / b / b;
		//if(fabs(((float)(m - 1)) - cr) < Consts.zero) goto L230;
		if(fabs(((float)(m)) - cr) < Consts.zero) goto L230;           //(JD 11/19/03)
		q2[m] = c / alfa / b * (cinf + ((REAL) 2.0 * cm2 * (REAL) sin(cmp) - cm) / (cm2 - (REAL) 0.25));
		sigma += alfa;
		cm += c;
		cmp += cpi;

	}

  goto L600;

L230:
  q2[m] = c / alfa / b * (cin(Consts.pi) + (REAL) 1.0);
  sigma += alfa;
  cm += c;
  cmp += cpi;
  is = m+1;
  goto L210;

L240:

  // UNKNOWN FILTER NAME
  fprintf(output, "\n           can not recognize filter name -- %s  job abolished", int2str(nf));
  return TRUE;

  // WRONG INPUT DATA

L400:
  fprintf(output, "\n           *****insufficient data items - job aborted*****");
  return TRUE;

  // JOB FINISHED

L600:
  fprintf(output, "\n           band-width = %7.3f * nyquist cutoff freq.", c);
  return FALSE;
}









   
