/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/foru_prjtrn.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 PRJTRN: PROJECTION TRANSFORM
 PURPOSE: READ NP'TH PROJECTION INTO VECTOR, PUT THE ANGLE IN
 PHI (IN RADIAN ) AND DPHI ( IN DEGREE )  WHERE THE ANGLE
 IS ALWAYS BETWEEN -90 AND 90
 IF THE ANGLE IS NOT IN THAT RANGE ADD N*PI OR SUBSTRACT
 N*PI TO MAKE IT IN THE RANGE AND REVERSE THE PROJECTION
 DATA IF NECESSARY
 THEN TRANSFORM & FILTER THE DATA
 ***NOTE THE UNCONVENTIONAL DEFINITION OF ANGLE AND ORDER OF PROJ
 ***DATA IN SNARK
 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "geom.h"
#include "spctrm.h"
#include "fourie.h"
#include "consts.h"
#include "uiod.h"
#include "anglst.h"
#include "projfile.h"

#include "foru.h"

void foru_class::prjtrn(INTEGER np, REAL* vector, REAL* phi, REAL* dphi)
{
	REAL theta;
	REAL sinth;
	REAL costh;
	static REAL oldth;

	REAL fact;
	INTEGER k;
	INTEGER invdir;

	BOOLEAN revers;

	Anglst.getang(np, &theta, &sinth, &costh);
	if (np != 0)
	{
		if (oldth > theta)
			error(3);
		if (GeoPar.uni && ((theta - oldth) > Consts.pid2))
			error(4);
		if (GeoPar.vri && ((theta - oldth) > (Consts.pi / 4.0)))
			error(4);
		if (Fourie.iflg == 1)
			return;
	}
	oldth = theta;
	*phi = theta - Consts.pid2;

	while (*phi < -Consts.pid2)
	{
		*phi += Consts.twopi;
	}

	while (*phi > (Consts.pi + Consts.pid2))
	{
		*phi -= Consts.twopi;
	}

	revers = (*phi > Consts.pid2);
	if (revers)
		*phi -= Consts.pi;
	*dphi = *phi * (REAL) 180.0 / Consts.pi;

	ProjFile.ReadProj(np, Fourie.ntemp, Fourie.nsize1);

	fact = Fourie.rinc;
	if (GeoPar.vri)
		fact = Fourie.rinc * (REAL) MAX0(fabs(cos(*phi)), fabs(sin(*phi)));
	if (GeoPar.strip)
	{
		for (k = 0; k < Fourie.nsize1; k++)
		{
			Fourie.ntemp[k] /= fact;
		}
	}

	ftmap(Fourie.ntemp, vector, revers);


	invdir = 1;
	fft(vector, Fourie.nforw, invdir, fact);

// Filtering
	frfilt(vector, Fourie.nsize1, Fourie.nfiltr, Fourie.cutoff, *phi);
	return;
}
