/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/geom.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************
 */

#include <cstdlib>
#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "geom.h"
#include "consts.h"
#include "uiod.h"

Geometry_class GeoPar;

Geometry_class::Geometry_class()
{
	nelem = 0;
	prjnum = 0;
	nave2 = 0;
	div = FALSE;
	tang = FALSE;
	arc = FALSE;
	par = FALSE;
	uni = FALSE;
	vri = FALSE;
	strip = FALSE;
	line = FALSE;
	//  rebinf = FALSE;    // bug 151 - swr - 08/07/2005
	lino = FALSE;  /// bug #135. Lajos, 08/01/2005
	radius = 0.0;
	stod = 0.0;
	aveden = 0.0;

	naper[0] = 1;
	for (int i = 1; i < 13; i++)
	{
		naper[i] = 0;
	}
}

/*
 COMPUTE THE NUMBER OF RAYS NECESSARY TO SPAN AN SQUARE OBJECT
 MELEN*ZISXIP UNITS ON A SIDE
 */

INTEGER Geometry_class::numray(INTEGER melen, REAL zisxip)
{
	REAL diag;
	REAL pmin;
	INTEGER numb;
	REAL xleg;

	diag = (REAL) (melen * zisxip / 1.414);

	pmin = diag / pinc;

	if (!div)
	{

		if (uni)
			numb = (INTEGER) (2.0 * pmin);
		if (vri)
			numb = (INTEGER) (2.0 * 1.414 * pmin);
	}
	else
	{
		xleg = (radius / pinc) * (radius / pinc) - pmin * pmin;
		if (xleg <= Consts.zero)
		{

			fprintf(output,
					"\n **** source to origin distance is %10.4f it must be at least %10.4f",
					radius, diag);
			fprintf(output, "\n **** program aborted\n");
			exit(700);
		}

		xleg = (REAL) sqrt(xleg);
		if (tang)
			numb = (INTEGER) (2.0 * stod * pmin / (xleg * pinc));
		if (arc)
			numb = (INTEGER) (2.0 * stod * atan2(pmin, xleg) / pinc);
	}

	return numb + (numb + 1) % 2 + 2;
}

void Geometry_class::print(char *comment)
{
	fprintf(output, "\n%s\n", comment);
	fprintf(output, "prjnum=%d, pinc=%f, nrays=%d, midray=%d, snrays=%d ", prjnum, pinc, nrays, midray, snrays);
	fprintf(output, "usrays=%d, fsnray=%d, lsnray=%d, fusray=%d, lusray=%d\n", usrays, fsnray, lsnray, fusray, lusray);
	fprintf(output, "div=%d, tang=%d, arc=%d, radius=%f, stod=%f ", div, tang, arc, radius, stod);
	fprintf(output, "par=%d, uni=%d, vri=%d, strip=%d, line=%d, lino=%d\n", par, uni, vri, strip, line, lino);  // bug 151 - swr - 08/07/2005
	fprintf(output, "aveden=%f nave2=%d naper=[%d", aveden, nave2, naper[0]);
	for (int i = 1; i < 13; i++)
	fprintf(output, ",%d", naper[i]);
	fprintf(output, "]\n");
};
