/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/get_phantom.c $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 Previously part of:

 /*--------------------------------------------------------------------------
 Super Snark V 1.0
 Written by Jingsheng Zheng... November 1992
 Modified by Jolyon Browne... June 1993
 ----------------------------------------------------------------------------*/

#include <string.h>
#include <stdlib.h>

#include "experimenter.h"
#include "errorc.h"
#include "get_phantom.h"

/*--------------------------------------------------------------------------
 This function reads the ensemble data file and returns a list of
 the phantom filenames.

 INPUTS:
 ensflname - name of ensemble data file.

 OUTPUTS:
 name - string array of length szens containing the list of phantom filenames.
 szens - number of phantom filenames in the ensemble data file.
 ----------------------------------------------------------------------------*/

void read_ensemble(char* ensflname, char name[][MAXLINESIZE], int* szens)
{
	int i = 0;
	char data[MAXLINESIZE];
	FILE *ensfl;

	if ((ensfl = fopen(ensflname, "r")) == NULL)
	{
		errorc("error in opening file", ensflname);
	}

	while (fgets(data, sizeof(data), ensfl) != NULL)
	{
		/* make sure 'i' is not over ensemble size limit */
		if (i >= MAXENSEMSIZE)
		{
			char limitmsg[25];
			sprintf(limitmsg, "(MAXENSEMSIZE = %d)", MAXENSEMSIZE);
			errorc("Exceeded maximum number of basic phantoms. ", limitmsg);
		}
		if (!isSkip(data))  //ignore comment lines and blank lines
			strcpy(name[i++], data);
	}

	*szens = i;
	fclose(ensfl);
}

/* -----------------------------------------------------------------

 INPUTS:
 iexp - experiment number.
 irun - run number.
 expflg - controls the phantom-file set used in each experiment as
 follows: if expflg=0 the same set of phantom files is available
 for use in both experiments. If expflg=1 the phantom files used
 in the first experiment are excluded in the second experiment.
 nphan - two-element array: nphan[i-1]= number of phantom files used for
 experiment i (i=1,2).
 size_ens - number of phantom filenames in name_lst.
 name_lst - string array of length szens_ens containing the list of phantom
 filenames.
 current_lst - string array of length phan[0] containing the list of
 phantom filenames to be used for experiment 1.
 

 OUTPUTS:
 size_ens - number of phantom filenames in name_lst.
 name_lst - string array of length szens_ens containing the list of phantom
 filenames.
 current_lst - string array of length phan[0] containing the list of
 phantom filenames to be used for experiment 1.
 phan_flname - the phantom filename randomly chosen.
 --------------------------------------------------------------------- */

void get_phantom(int iexp, int irun, int expflg, int* nphan, int* size_ens, char name_lst[][MAXLINESIZE], char current_lst[][MAXLINESIZE], char* phan_flname)
{
	int i, j, szens = *size_ens;

	if (expflg == 1 && iexp == 0 && irun == 0)
	{
		for (j = 0; j < nphan[iexp]; j++)
		{

			i = (int) (szens * drand48()); /* (a) */
			strcpy(current_lst[j], name_lst[i]); /* (b) */
			strcpy(name_lst[i], name_lst[szens - 1]); /* (c) */
			szens--; /* (d) */
		}

		i = (int) (nphan[iexp] * drand48()); /* randomly select a phantom */
		strcpy(phan_flname, current_lst[i]); /* filename from current_lst */
		phan_flname[strlen(phan_flname) - 1] = '\0';
	}
	else if (expflg == 1 && iexp == 0 && irun > 0)
	{

		i = (int) (nphan[iexp] * drand48());
		strcpy(phan_flname, current_lst[i]);
		phan_flname[strlen(phan_flname) - 1] = '\0';
	}
	else
	{

		i = (int) (szens * drand48());
		strcpy(phan_flname, name_lst[i]);
		phan_flname[strlen(phan_flname) - 1] = '\0';
	}
	*size_ens = szens; /* number of phantom filenames in name_lst */
}
