/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/pick.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 EACH CALL TO PICK RETURNS THE NUMBER NR OF A NEW RAY AND THE
 NUMBER NP OF A NEW PROJECTION ACCORDING TO THE SELECTION OF
 SEQUENCING MADE BY THE SELECT COMMAND.  DEFAULT IS SEQUENTIAL RAY
 SELECTION WITH RAY-STEP = PROJECTION-STEP = 1
 */

#include <cstdio>

#include "blkdta.h"
#include "geom.h"
#include "raysel.h"
#include "uiod.h"
#include "geom.h" 
#include "effpick.h" 

#include "DIGRand.h"

#include "pick.h"

void pick(INTEGER* np, INTEGER* nr)
{

	switch (RaySel.stype)
	{

	//EFFICIENT RAY PICKING

	case 7: // efficient picking

		if (RaySel.effcnt % (RaySel.numray * GeoPar.prjnum) == 0)
		{
			RaySel.old_numprj[0] = -1;
			RaySel.effcnt = 0; // Bug #214, wei, 11/20/2006
		}

		if (RaySel.effcnt % RaySel.numray == 0)
		{
			RaySel.old_numray[0] = -1;

			pi_map(RaySel.num_digprj, RaySel.decompprj, RaySel.baseprj,
					&RaySel.old_numprj, &RaySel.new_numprj, &RaySel.curnp);
		}

		pi_map(RaySel.num_digray, RaySel.decompray, RaySel.baseray,
				&RaySel.old_numray, &RaySel.new_numray, &RaySel.curnr);

		if (!RaySel.useray)
		{
			RaySel.curnr += GeoPar.fsnray; //(GeoPar.nrays-GeoPar.snrays)/2;
		}
		else
		{
			RaySel.curnr += GeoPar.fusray; //(GeoPar.nrays-GeoPar.usrays)/2;
		}

		RaySel.effcnt++;

		break;

		// RANDOM
	case 2:
		L30: if (!RaySel.useray)
		{
			RaySel.curnr = (INTEGER) (Rand() * ((REAL) (GeoPar.snrays))
					+ ((REAL) (RaySel.begray))); // bug 184 - swr - 11/04/05
		}
		else
		{
			RaySel.curnr = (INTEGER) (Rand() * ((REAL) (GeoPar.usrays))
					+ ((REAL) (RaySel.begray))); // bug 184 - swr - 11/04/05
		}
		RaySel.curnp = (INTEGER) (Rand() * (REAL) (GeoPar.prjnum)); // bug 184 - swr - 11/04/05
		break;

		// SEQP
	case 1:
		L40: RaySel.curnp += RaySel.prjinc;
		if (RaySel.curnp >= GeoPar.prjnum)
		{
			RaySel.curnp = RaySel.initnp;
			RaySel.curnr += RaySel.rayinc;
			if (RaySel.curnr > RaySel.endray)
			{
				RaySel.initnr++;
				if (RaySel.initnr
						>= MIN0((RaySel.begray + RaySel.rayinc - 1),
								RaySel.endray))
				{
					RaySel.initnr = RaySel.begray;
					RaySel.initnp++;
					if (RaySel.initnp >= MIN0(RaySel.prjinc, GeoPar.prjnum))
					{
						RaySel.initnp = 0;
					}
					RaySel.curnp = RaySel.initnp;
				}
				RaySel.curnr = RaySel.initnr;
			}
		}
		break;

		// SEQR
	case 0:
		L50: RaySel.curnr += RaySel.rayinc;
		if (RaySel.curnr > RaySel.endray)
		{
			RaySel.curnr = RaySel.initnr;
			RaySel.curnp += RaySel.prjinc;
			if (RaySel.curnp >= GeoPar.prjnum)
			{
				RaySel.initnp++;
				if (RaySel.initnp >= MIN0(RaySel.prjinc, GeoPar.prjnum))
				{
					RaySel.initnp = 0;
					RaySel.initnr++;
					if (RaySel.initnr
							>= MIN0((RaySel.begray + RaySel.rayinc - 1),
									RaySel.endray))
					{
						RaySel.initnr = RaySel.begray;
					}
					RaySel.curnr = RaySel.initnr;
				}
				RaySel.curnp = RaySel.initnp;
			}
		}
		break;

		// (CYCLES) OF RAYS AT RANDOM FOLLOWED BY SEQUENTIAL RAYS
	case 3:
		if (RaySel.rcnt1 == RaySel.rn1)
		{
			if (RaySel.rn2 <= 0)
				goto L50;
			if (RaySel.rcnt2 != RaySel.rn2)
			{
				RaySel.rcnt2++;
				goto L50;
			}
			RaySel.rcnt1 = 0;
			RaySel.rcnt2 = 0;
		}
		RaySel.rcnt1++;
		goto L30;

		// (CYCLES) OF RAYS AT RANDOM FOLLOWED BY SEQUENTIAL
		// PROJECTION SELECTION
	case 4:
		if (RaySel.rcnt1 == RaySel.rn1)
		{
			if (RaySel.rn2 <= 0)
				goto L40;
			if (RaySel.rcnt2 != RaySel.rn2)
			{
				RaySel.rcnt2++;
				goto L40;
			}
			RaySel.rcnt1 = 0;
			RaySel.rcnt2 = 0;
		}
		RaySel.rcnt1++;
		goto L30;

		// (CYCLES) OF SEQUENTIAL RAYS FOLLOWED BY RAYS AT RANDOM
	case 5:
		if (RaySel.rcnt1 == RaySel.rn1)
		{
			if (RaySel.rn2 <= 0)
				goto L30;
			if (RaySel.rcnt2 != RaySel.rn2)
			{
				RaySel.rcnt2++;
				goto L30;
			}
			RaySel.rcnt1 = 0;
			RaySel.rcnt2 = 0;
		}
		RaySel.rcnt1++;
		goto L50;

		// (CYCLES) OF SEQUENTIAL PROJECTION RAYS FOLLOWED BY RAYS AT RANDOM
	case 6:
		if (RaySel.rcnt1 == RaySel.rn1)
		{
			if (RaySel.rn2 <= 0)
				goto L30;
			if (RaySel.rcnt2 != RaySel.rn2)
			{
				RaySel.rcnt2++;
				goto L30;
			}
			RaySel.rcnt1 = 0;
			RaySel.rcnt2 = 0;
		}
		RaySel.rcnt1++;
		goto L40;
	}

	// COMMON RETURN
	*np = RaySel.curnp;
	*nr = RaySel.curnr;

	if (trace > 4)
	{
		fprintf(output, "\n          pick    np = %5i  nr = %5i", *np, *nr);
	}
	return;
}

