/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/pix2blob.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************
 */

/* This subroutine converts an image function in pixel basis into that in blob basis. This subroutine uses SIRT type algorithm with blob footprints for speeding up. See subroutine blob2pix() for the meaning of various variables.
 */

#include <cstdio>
#include <cmath>
#include <cstdlib>

#include "blkdta.h"
#include "blob.h"
#include "geom.h"
#include "consts.h"
#include "effpick.h" 
#include "uiod.h"

void Blob_class::pix2blob(REAL* pict, REAL* blob_array)
{
	INTEGER m, h, m1, h1;
	INTEGER i, k, j, ii, kk, jj;
	INTEGER jc, kc;
	INTEGER M2, H2;
	INTEGER nelem2;
	INTEGER jx, ky;
	INTEGER ind, indb;


	REAL xc, yc, auy;
	REAL ipixsiz = (REAL) 1.0 / GeoPar.pixsiz;
	INTEGER count, ncount;
	REAL maxpict, minpict;
	REAL amppict;
	REAL nc;
	REAL sd;
	INTEGER niter;
	REAL width;
	REAL rneigh;
	REAL tau;
	REAL unif;
	REAL frelax;

	frelax = 2.0; //changed on 4/8/04, from 1.0 to 2.0, as suggested by Laszlo
	tau = 0.3;
	unif = 0.0005;
	width = 3.5;
	rneigh = 0.75;
	niter = 5000;

	frelax *= GeoPar.pixsiz * GeoPar.pixsiz
			/ (Consts.pi * Blob.support * Blob.support);
	frelax = MIN0(2.0, frelax);  //frelax param cannot be greater than 2 !

	REAL *u = new REAL[GeoPar.area]; //art4 dual var
	REAL *Delta = new REAL[GeoPar.area]; //art4 epsilon
	REAL *Gamma = new REAL[GeoPar.area]; //art4 epsilon
	INTEGER *que = new INTEGER[GeoPar.area]; //art4

	INTEGER pow2, num_dig;
	INTEGER *decomp = NULL;
	INTEGER *base = NULL;      //wei 3/2005
	INTEGER *old_numk, *new_numk;
	INTEGER *old_numj, *new_numj;

	H2 = (INTEGER) (Blob.H / 2);
	M2 = (INTEGER) (Blob.M / 2);
	nelem2 = (INTEGER) (GeoPar.nelem / 2);

	//get the decomposition of the smallest power of 2 >= nelem
	pow2 = 1;

	while (pow2 < GeoPar.nelem)
	{
		pow2 *= 2;
	}

	factorization(pow2, &decomp, &num_dig);
	get_base(decomp, num_dig, &base);
	old_numk = new INTEGER[num_dig];
	new_numk = new INTEGER[num_dig];
	old_numj = new INTEGER[num_dig];
	new_numj = new INTEGER[num_dig];

	// initialising the blobs with the value of the nearest pixel and computing mod2 using BLOB footprint

	for (h = lhf; h <= lhl; h++)
	{
		for (m = lmf; m <= lml; m++)
		{ // m and h must have the same parity!!
			if ((m + h) % 2 == 0)
			{
				m1 = m + M2;
				h1 = h + H2;

				indb = h1 * Blob.M + m1;

				xc = di2 * m;  // coord of the center of the blob (m,h)
				yc = di2sq3 * h;

				auy = ipixsiz * yc;
				jx = (INTEGER) floor(ipixsiz * xc + 0.5); // coord of the center of
				ky = (INTEGER) floor(auy + 0.5);        // the nearest pixel

				jc = jx + nelem2; // bring to the right address
				kc = (INTEGER) floor(-auy + 0.5) + nelem2;

				if (jc > ljl)
					jc = ljl;
				if (jc < ljf)
					jc = ljf;

				if (kc > lkl)
					kc = lkl;
				if (kc < lkf)
					kc = lkf;

				blob_array[indb] = pict[kc * GeoPar.nelem + jc];
			}
		}
	}

	//initialize dual var and finding the max amplitud between pict and 0
	maxpict = -Consts.infin;
	minpict = Consts.infin;
	for (k = 0; k < GeoPar.nelem; k++)
	{
		for (j = 0; j < GeoPar.nelem; j++)
		{
			ind = k * GeoPar.nelem + j;
			if (maxpict < pict[ind])
			{
				maxpict = pict[ind];
			}
			if (minpict > pict[ind])
			{
				minpict = pict[ind];
			}
			u[ind] = 0.0;
		}
	}
	amppict = MAX0(maxpict,0.0) - MIN0(minpict, 0.0);

	//computing delta and gamma of ART4
	for (k = 0; k < GeoPar.nelem; k++)
	{
		for (j = 0; j < GeoPar.nelem; j++)
		{
			ind = k * GeoPar.nelem + j;
			p2b_neighb(k, j, pict, &nc, rneigh);
			if (pict[ind] >= nc)
			{
				Delta[ind] = unif * amppict;
				Gamma[ind] = -(tau * fabs(pict[ind] - nc) + unif * amppict);
			}
			else
			{
				Delta[ind] = tau * fabs(pict[ind] - nc) + unif * amppict;
				Gamma[ind] = -unif * amppict;
			}
		}
	}

	i = 0;
	for (;;)
	{
		//initializing list "que" and count
		count = 0;
		old_numk[0] = -1;
		for (kk = 0; kk < pow2; kk++)
		{
			pi_map(num_dig, decomp, base, &old_numk, &new_numk, &k);
			if (k < GeoPar.nelem)
			{
				old_numj[0] = -1;
				for (jj = 0; jj < pow2; jj++)
				{
					pi_map(num_dig, decomp, base, &old_numj, &new_numj, &j);
					if (j < GeoPar.nelem)
					{
						ind = k * GeoPar.nelem + j;
						que[count] = ind;
						count++;
					}
				} //for jj
			}
		} //for kk
		  //ART4-type algorithm
		ii = 0;
		for (;;)
		{
			Correct(pict, blob_array, u, Delta, Gamma, que, count, &ncount, &sd,
					width, frelax);
			count = ncount;

			if (count == 0 || i >= niter)
				break;
			i++;
			ii++;
		}
		if (ii == 0 || i >= niter)
			break;
	}

	delete[] old_numk;  // bug 92 - Lajos - 03/02/2005
	delete[] new_numk;  // bug 92 - Lajos - 03/02/2005
	delete[] old_numj;  // bug 92 - Lajos - 03/02/2005
	delete[] new_numj;  // bug 92 - Lajos - 03/02/2005
	delete[] u;  // bug 92 - Lajos - 03/02/2005
	delete[] Delta;  // bug 92 - Lajos - 03/02/2005
	delete[] Gamma;  // bug 92 - Lajos - 03/02/2005
	delete[] que;  // bug 92 - Lajos - 03/02/2005
	if (decomp != NULL)
		delete[] decomp;
	if (base != NULL)
		delete[] base;      //wei 3/2005

	return;
}

void Blob_class::Correct(REAL* pict, REAL* blob_array, REAL* u, REAL* Delta,
		REAL* Gamma, INTEGER* que, INTEGER count, INTEGER* ncount, REAL* sd,
		REAL width, REAL frelax)
{

	INTEGER ind, indb;
	INTEGER c;
	INTEGER k, j;
	INTEGER numb;
	INTEGER multp = (INTEGER) floor(Blob.support / Blob.delta + 0.5) + 1;
	INTEGER npts = 4 * multp * multp;
	INTEGER *list = new INTEGER[npts];
	INTEGER g;
	REAL *weight = new REAL[npts];
	REAL sum;
	REAL snorm;
	REAL diff;
	REAL lowb, uppb;
	REAL ck;

	*sd = 0;
	*ncount = 0;

	for (c = 0; c < count; c++)
	{
		ind = que[c];
		sum = 0.0;
		k = ind / GeoPar.nelem;
		j = ind % GeoPar.nelem;
		bpix(k, j, list, weight, &numb, &snorm);
		if (numb > 0)
		{
			for (g = 0; g < numb; g++)
			{
				indb = list[g];
				sum += blob_array[indb] * weight[g];
			}
			diff = pict[ind] - sum;
			(*sd) += (diff * diff);
			if (!(diff <= -width * Gamma[ind])
					|| !(diff >= -width * Delta[ind])) //Ran bug 235 August 17, 2008
			{
				que[(*ncount)] = ind;
				(*ncount)++;
				//compute the change
				lowb = frelax * (diff + Gamma[ind]) / snorm;
				uppb = frelax * (diff + Delta[ind]) / snorm;
				ck = MAX0(MIN0(uppb, u[ind]), lowb);

				// updates of dual var
				u[ind] -= ck;
				// backward operation with relaxation param relax
				for (g = 0; g < numb; g++)
				{
					indb = list[g];
					blob_array[indb] += (ck * weight[g]);
				}
			} // if no satisf
		} //if numb
	} //if c
	delete[] list;  // bug 92 - Lajos - 03/02/2005
	delete[] weight;  // bug 92 - Lajos - 03/02/2005
	return;
}

void Blob_class::p2b_neighb(INTEGER k, INTEGER j, REAL* pict, REAL *nc,
		REAL rneigh)
{

	INTEGER m, n;
	INTEGER kk, jj, ind;
	INTEGER size;
	REAL mult = rneigh * Blob.support / GeoPar.pixsiz;
	INTEGER multn = (INTEGER) ceil(mult);

	size = 0;
	(*nc) = 0.0;

	for (m = -multn; m <= multn; m++)
	{
		for (n = -multn; n <= multn; n++)
		{
			if ((n * n + m * m) <= (mult * mult))
			{
				size++;
			}
		}
	}
	for (m = -multn; m <= multn; m++)
	{
		kk = k + m;
		if (kk >= 0 && kk < GeoPar.nelem)
		{
			for (n = -multn; n <= multn; n++)
			{
				jj = j + n;
				if (jj >= 0 && jj < GeoPar.nelem)
				{
					if ((n * n + m * m) <= (mult * mult))
					{
						ind = kk * GeoPar.nelem + jj;
						(*nc) += pict[ind];
					}
				}
			}
		}
	}
	(*nc) /= (REAL) size;

	return;
}

