/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/pointwse.c $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 Previously part of SuperSNARK
 */

#include "experimenter.h"
#include "read_eval_recon2.h"
#include "pointwse.h"

/* ----------------------------- pointwse.c -------------------------------

 This function computes the pointwise accuracy.

 INPUTS:
 mode - number of MODE commands read in so far.
 itr1 - array of length niters containing the iteration numbers for the
 first algorithm.
 itr2 - array of length niters containing the iteration numbers for the
 second algorithm.
 niters - number of iterations to be compared
 keywrd1 - string containing the keyword which defines the first algorithm.
 keywrd2 - string containing the keyword which defines the second algorithm.

 OUTPUTS:
 str_acc1 - array of length niters containing the structural accuracy for
 the first algorithm.
 str_acc1 - array of length niters containing the structural accuracy for
 the second algorithm.

 */

void pointwse(int mode, int* itr1, int* itr2, int niters, char* keywrd1, char* keywrd2, double* poin_acc1, double* poin_acc2)

{
	double recon;
	int i;


	for (i = 0; i < niters; i++)
	{ /* read in normalized root mean square distance for
	 the reconstructions. */

		read_eval_recon2(mode, itr1[i], keywrd1, &recon);
		poin_acc1[i] = -recon;
		read_eval_recon2(mode, itr2[i], keywrd2, &recon);
		poin_acc2[i] = -recon;

	}

}
