/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/projection.c $
 $LastChangedRevision: 118 $
 $Date: 2014-07-09 14:22:29 -0400 (Wed, 09 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 Previously part of:
 /*--------------------------------------------------------------------------
 Super Snark V 1.0
 Written by Jingsheng Zheng... November 1992

 Modified by Jolyon Browne... June 1993
 ----------------------------------------------------------------------------*/

#include <strings.h>

#include "experimenter.h"
#include "errorc.h"
#include "projection.h"

/*--------------------------------------------------------------------------

 This function echos the information stored in the file specified on
 line 3 of the SuperSNARK input sequence. This information is sent to the
 SNARK input stream.

 INPUTS:
 proj_flname - name of file containing information about generation of the
 projection data. This is the file specified on line 3 of the
 SuperSNARK input sequence.
 seed - seed used in the random number generator.

 OUTPUTS:
 none.
 ----------------------------------------------------------------------------*/

/**
 * 20140618 - BPRO: BUG 288 QUICK-FIX
 *
 * Replace current version of void projection(char* proj_flname, long seed)
 * with the version from tar-ball: snark14.11.02.23
 *
 * Reason:
 * The current version (tar-ball: snark14.14.04.10) should verify the
 * input file of the projection command regarding its validity, but is missing
 * some options (e.g. NITERS).
 * The old version (tar-ball: snark14.11.02.23) does not perform any checks but
 * copies all lines into the snark_e[runNo].in file.
 *
 * The file projection.c of the current version has been added as
 * projection_snark14.14.04.10.c
 * to SVN --> it can be used as basis for a "real" bug fix of bug 288
 */
void projection(char* proj_flname, long seed)
{
	char string[MAXLINESIZE];
	FILE *projfl;

	//seed = -1; // bug 176 - swr - 11/8/05
	// bug 269, Joanna Klukowska, Feb. 13, 2011
	// projection seed is generated randomly based on the seed in the input file
	long projection_seed = (long) (100000000 * drand48());

	if ((projfl = fopen(proj_flname, "r")) == NULL)
	{
		errorc("error in opening file", proj_flname);
	}

	while ((fgets(string, sizeof(string), projfl)) != NULL)
	{
		while (isSkip(string)) //it's a comment line,
		{                    //echo it to output and read next line
			if (isComment(string))
				fprintf(pstream, "%s", string);
			if (fgets(string, sizeof(string), projfl) == NULL)
				break;
		}
		if (strncasecmp(string, "seed", 4) == 0)
		{
			fprintf(pstream, "seed %ld\n", projection_seed);
		}
		else
		{
			fprintf(pstream, "%s", string);
		}
	}

	fclose(projfl);
}
