/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/quad_matrix.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************


 THIS ROUTINE CALCULATES

 Y = D*M#*A*M*X     IF COPT .LE. 3

 OR

 Y = C**P*(C*(SA+M#*A*M + SB*B**C) + SC)*X   OTHERWISE

 WHERE P = T - 1
 */

#include <cstdio>

#include "blkdta.h"
#include "geom.h"
#include "consts.h"
#include "blob.h"

#include "quad.h"

void quad_class::matrix(REAL* y, REAL* x, REAL* d, INTEGER* list, REAL* weight,
		REAL* s)
{
	REAL* j;
	INTEGER i;
	INTEGER area;

	if (Blob.pix_basis)
	{
		area = GeoPar.area;
	}
	else
	{
		area = Blob.area;
	}

	// COMPUTE SA*M#*A*M*X STORE IN Y

	if (sa > Consts.zero)
		mtamx(x, y, list, weight);

	if (copt > 3)
	{

		if (sb > Consts.zero)
		{

			j = new REAL[area];

			for (i = 0; i < area; i++)
			{
				j[i] = x[i];
			}

			if (l > 0)
			{
				if (Blob.pix_basis)
				{
					adsmos(j, l, bw1, bw2, bw3, bbcon);
				}
				else
				{
					badsmos(j, l, bw1, bw2, bbcon);
				}
			}
			for (i = 0; i < area; i++)
			{
				y[i] += sb * j[i];
			}
			delete[] j;  // bug 92 - Lajos - 03/02/2005
		}
		if (copt != 5)
		{
			if (Blob.pix_basis)
			{
				adsmos(y, 1, cw1, cw2, cw3, cbcon);
			}
			else
			{
				badsmos(y, 1, cw1, cw2, cbcon);
			}
		}

		if (sc != 0.0)
		{

			for (i = 0; i < area; i++)
			{
				y[i] += sc * x[i];
			}
		}

		if ((copt != 5) && ((t - 1) > 0))
		{
			if (Blob.pix_basis)
			{
				adsmos(y, t - 1, cw1, cw2, cw3, cbcon);
			}
			else
			{
				badsmos(y, t - 1, cw1, cw2, cbcon);
			}
		}

		if (gopt > 0)
		{
			for (i = 0; i < area; i++)
			{
				y[i] *= s[i];
			}
		}

		if ((copt != 5) && (t > 0))
		{
			if (Blob.pix_basis)
			{
				adsmos(y, t, cw1, cw2, cw3, cbcon);
			}
			else
			{
				badsmos(y, t, cw1, cw2, cbcon);
			}
		}
		return;
	}
	// D SECTION

	for (i = 0; i < area; i++)
	{
		y[i] *= d[i];
	}

	return;
}
