/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/snark.cpp $
 $LastChangedRevision: 177 $
 $Date: 2015-08-03 16:44:07 -0400 (Mon, 03 Aug 2015) $
 $Author: zye $
 ***********************************************************

 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                               *
 *                         S N A R K   9 3                       *
 *                                                               *
 *                A PICTURE RECONSTRUCTION PROGRAM               *
 *                                                               *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

             WRITTEN- DECEMBER 1974
             STUART W. ROWLAND
             ROGER W. PERETTI
             DEPT. OF COMPUTER SCIENCE
             S.U.N.Y. AT BUFFALO
             4226 RIDGE LEA ROAD
             AMHERST, NEW YORK     14226
             TEL. (716) 831-1351

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

             REVISED BY-  G. T. HERMAN
                          K.J. CHEN
             SUMMER 1975

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

   SNARK76 - PRELIMINARY VERSION - SPRING 1976
THIS VERSION OF SNARK INCORPORATES A NUMBER OF IMPROVEMENTS TO
THE SNARK75 PROGRAM.  THE MOST SIGNIFICANT OF WHICH ARE-
   1) THE ABILITY TO GENERATE PROJECTION DATA SIMULATING A
      POLYCHROMATIC X-RAY SOURCE.
   2) THE INTRODUCTION OF AN OVERLAY STRUCTURE GREATLY REDUCING
      THE MEMORY REQUIREMENTS OF SNARK.  ALSO, CREATR AND SKUNK
      HAVE BEEN MODIFIED SO THAT THEY ARE CONSIDERABLY SMALLER.
   3) COMMENTS AND MESSAGES HAVE BEEN MODIFIED SO THAT THEY ARE
      MORE UNDERSTANDABLE.
   4) NON-STANDARD SUBSCRIPTING HAS BEEN REMOVED.
   5) ONLY THE RECONSTRUCTION FILE (RECFIL) NEED BE PRESENT DURING
      POST-PROCESSING FOR MOST OPERATIONS.  THE EXCEPTION IS THE
      DISTANCE COMMAND WHICH REQUIRES THE PROJECTION FILE (PRJFIL)
      TO COMPUTE THE RESIDUAL.
   6) THE ABILITY TO SPECIFY THE RANDOM NUMBER GENERATOR SEED HAS
      BEEN ADDED TO CREATE.
   7) A SIMPLE PHOTON SCATTERING MODEL HAS BEEN ADDED TO CREATE.
THESE CHANGES HAVE BEEN IMPLEMENTED BY-
S. W. ROWLAND AND G. T. HERMAN.

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

   SNARK77 - AUTUMN 1977
THIS MODIFICATION OF SNARK76 PROVIDES A SIMPLER AND MORE POWERFUL
COMMAND LANGUAGE STRUCTURE.  THE MAJOR MODIFICATIONS ARE:
   1) THE ADDITION OF THE ROUTINES GETINT AND GETNUM TO EXPAND THE
      CAPABILITIES OF THE COMMAND LANGUAGE.  GETINT REPLACES THE
      INTEGER OPTION WHICH WAS PART OF THE OLD GETWRD.  GETNUM
      PROVIDES A FREE FORM FLOATING POINT INPUT FACILITY TO
      SNARK77.
   2) THE CREATE (AND THEREFORE PICTURE AND PROJECTION) INPUT
      SEQUENCE HAS BEEN MODIFIED SO THAT IT IS MORE UNDERSTANDABLE
      WITHOUT REFERENCING THE SNARK MANUAL.  ALL OF THE INFORMATIO
      USED TO CREATE A PHANTOM OR PROJECTION DATA IS RETAINED IN
      THE GENERATED DATA SET HEADERS SO THAT THIS INFORMATION WILL
      NOT BE LOST TO FUTURE USERS OF THE DATA SET.
   3) A NAME NOW MUST BE ASSIGNED TO EACH ALGORITHM EXECUTION.
      THIS NAME IS PRINTED ON ALL POST-PROCESSING OUTPUT TO IDENTI
      THE PARTICULAR RECONSTRUCTION BEING USED.  IT IS SUGGESTED
      THAT THIS CARD INCLUDE AN INDICATION OF THE PARAMETERS USED
      BY THE ALGORITHM.
   4) ALMOST ALL OF THE SNARK CONTROL CARD FORMATS HAVE CHANGED.
   5) A CAPABILITY TO GENERATE A PARALLEL PROJECTION DATA SET FROM
      A DIVERGENT PROJECTION DATA SET VIA THE REBINNING FACILITY
      OF RDPROJ HAS BEEN ADDED.
   6) THE LINES COMMAND HAS BEEN EXTENDED TO GENERATE THE OUTPUT
      ON A DIGITAL PLOTTER.

   THE OVERLAY STATEMENTS FROM THE CDC3500 VERSION OF SNARK76 HAVE
BEEN REMOVED FOR THIS BURROUGHS B1700 VERSION.  SINCE THE SIZE OF
THE PROGRAM HAS INCREASED TREMENDOUSLY, OVERLAYING WILL HAVE TO BE
INSTALLED ON NON-PAGING COMPUTERS.  THE FOLLOWING TWO-LEVEL
STRUCTURE IS SUGGESTED:
   (0,0)- THE MAIN PROGRAM AND COMMON SUBROUTINES.
   (1,0)- THE CREATER SUBSYSTEM.
   (2,0)- RDPICT AND RDPROJ.
   (3,0)- EXALG.
   (3,N)- EACH OF THE RECONSTRUCTION ALGORITHMS.
   (4,0)- THE POST-PROCESSING ROUTINES.
EVEN WITH THIS STRUCTURE, IT MAY BE THAT SOME OF THE LARGER
ALGORITHMS (QUAD, FOR EXAMPLE) MAY NOT FIT IN YOUR LIMITED
MEMORY AND SHOULD THEREFORE BE DELETED FROM THE SYSTEM.

THESE CHANGES HAVE BEEN IMPLEMENTED BY-
S. W. ROWLAND AND THE MEDICAL IMAGE PROCESSING GROUP (MIPG).

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

   SNARK89 Version 1.01 - July 1990
The calculation of totden and totlen in the STRIP case in rdproj.f
has been changed to agree with the manual.  Checks of nelem and
pixsiz have been inserted in rdpict.f.
These changes were made by G.T. Herman and D. Odhner.

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

SNARK05

The major advances that are incorporated into SNARK05 are:

1. SNARK05 is implemented in C++;
2. the file structures for holding projection data, phantoms and
   reconstructions have been redesigned to include XML headers;
3. all itertive algorithms are now capable of handling image 
   representations that use "blobs" as well as "pixels";
4. the efficient data access ordering is a standard feature;
5. all artificial restrictions on sizes have been removed - the only
   remaining restrictions are those imposed by the hardware, compiler
   and operating system.

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

MAIN PROGRAM FOR SNARK SYSTEM
THIS ROUTINE READS THE SNARK CONTROL CARDS, INTERPRETS THE MAJOR
COMMAND ON THE CARD, AND INVOKES THAT ROUTINE.

*/      

#include <cstdio>
#include <string>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "blkdta.h"
#include "uiod.h"

#include "basis.h"   
#include "second.h"
#include "mode.h"
#include "settra.h"
#include "creatr.h"
#include "rdpict.h"
#include "rdproj.h"
#include "select.h"
#include "stopex.h"
#include "exalg.h"
#include "eval.h"
#include "disply.h"
#include "pnch.h"
#include "doline.h"
#include "halft.h"
#include "settra.h"
#include "infile.h" 
#include "recfile.h"
#include "file11.h"
#include "int2str.h"
#include "objects.h"
#include "superior.h"

/*
  "JUST THE PLACE FOR A SNARK!' THE BELLMAN CRIED,
       AS HE LANDED HIS CREW WITH CARE;
  SUPPORTING EACH MAN ON THE TOP OF THE TIDE
       BY A FINGER ENTWINED IN HIS HAIR.
  "JUST THE PLACE FOR A SNARK!  I HAVE SAID IT TWICE:
       THAT ALONE SHOULD ENCOURAGE THE CREW.
  JUST THE PLACE FOR A SNARK!  I HAVE SAID IT THRICE:
       WHAT I TELL YOU THREE TIMES IS TRUE."
  FIT THE FIRST - THE LANDING
  THE HUNTING OF THE SNARK (AN AGONY IN EIGHT FITS)
  BY LEWIS CARROLL
*/

#define NUMBER_OF_COMMANDS 16

enum SnarkCommands
{
  SNARK_COMMAND_MODE,
  SNARK_COMMAND_TRACE,
  SNARK_COMMAND_END,
  SNARK_COMMAND_CREATE,
  SNARK_COMMAND_PICTURE,
  SNARK_COMMAND_PROJECTION,
  SNARK_COMMAND_BASSIS,
  SNARK_COMMAND_SELECT,
  SNARK_COMMAND_STOP,
  SNARK_COMMAND_EXECUTE,
  SNARK_COMMAND_EVALUATE,
  SNARK_COMMAND_DISPLAY,
  SNARK_COMMAND_PUNCH,
  SNARK_COMMAND_LINES,
  SNARK_COMMAND_SKUNK,
  SNARK_COMMAND_SUPERIORIZE
};

static const INTEGER main_codes[NUMBER_OF_COMMANDS] = 
{
  CHAR2INT('m','o','d','e'),
  CHAR2INT('t','r','a','c'),
  CHAR2INT('e','n','d',' '),
  CHAR2INT('c','r','e','a'),
  CHAR2INT('p','i','c','t'),
  CHAR2INT('p','r','o','j'),
  CHAR2INT('b','a','s','i'),  
  CHAR2INT('s','e','l','e'),
  CHAR2INT('s','t','o','p'),
  CHAR2INT('e','x','e','c'),
  CHAR2INT('e','v','a','l'),
  CHAR2INT('d','i','s','p'),
  CHAR2INT('p','u','n','c'),
  CHAR2INT('l','i','n','e'),
  CHAR2INT('s','k','u','n'),
  CHAR2INT('s','u','p','e')
};

int FindKeywordCode(int word)
{
  for(int i = 0; i < NUMBER_OF_COMMANDS; i++) {       // find keyword code
    if(word == main_codes[i]) {
      return i;
    }
  }
  return -1;
}

INTEGER GetCommandPhase(INTEGER code)
{
  static const INTEGER Phase[NUMBER_OF_COMMANDS] =
  {
//    0, // *     Comment
    0, // mode  Mode
    0, // trac  Trace 
    0, // end   End
    1, // crea  Create
    2, // pict  Picture
    2, // proj  Projection
    3, // basi  Basis
    3, // sele  Select
    3, // stop  Stop
    3, // exec  Execue
    4, // eval  Evaluate
    4, // disp  Display
    4, // punc  Punch
    4, // line  Lines
    4, // skun  Skunk
    3  // supe  Superiorize
  };

  if(code >= NUMBER_OF_COMMANDS) {
    return 4;
  }

  return Phase[code];
}


int snark(int argc, char *argv[])
{
  ///static const INTEGER syst = CHAR2INT('a','y','s','t');
  //static const INTEGER snar = CHAR2INT('s','n','a','r');
  //static const INTEGER rand_1 = CHAR2INT('r','a','n','d');

  INTEGER word;
  BOOLEAN eol;
  //BOOLEAN flag;
  INTEGER lphase;  // last phase
  INTEGER phase;   // current phase
  BOOLEAN iptflg;   // picture command called
  BOOLEAN iprflg;   // projection command called
  
  BOOLEAN islflg;   // select command called, by wei 1/05 

  REAL begin;
  REAL curr_time;
  REAL dur;
  int CommandCode;

  // bug 179 - swr - 10/30/05
#ifndef __CYGWIN__
  // bug 167 - swr - 9/24/05
  int lfp=open("snark.lock",O_RDWR|O_CREAT,0640);
  if (lfp<0) {
    fprintf(stderr, "can't create lock file in current directory - check directory permissions\n");
    exit(1); /* can not open */
  }
  if (lockf(lfp,F_TLOCK,0)<0) {
    fprintf(stderr, "multiple snark executions in the same directory are not allowed!\n");
    exit(0); /* can not lock */
  }
  /* only first instance continues */
  char str[10];
  sprintf(str,"%d\n",getpid());
  write(lfp,str,strlen(str)); /* record pid to lockfile */
#endif

  // bug 187 - allow input filename on command line - swr - 11/11/05
  if (argc > 1) {
    close(fileno(stdin));
    if (open(argv[1], O_RDONLY) < 0) {
      perror(argv[1]);
      exit(1);
    }
  }

  InFile.Open();

  iptflg = FALSE;
  iprflg = FALSE;
  islflg = FALSE;     //initialization islflg, by wei 1/05
 
  std::string progName = argv[0];    // bug89 - fix header - swr - 2/25/05
  
  progName = progName.substr(progName.find_last_of('/')+1);

  fprintf(output, "   %s.s170710 - A PICTURE RECONSTRUCTION PROGRAM\n", progName.c_str());

  lphase = 0;

  second(&begin);

  for(;;) {      

    InFile.getnxt(TRUE);        // get next line of input  // changed "FALSE" to "TRUE. Lajos, Jan 25, 2005

    word = InFile.getwrd(FALSE, &eol, main_codes, NUMBER_OF_COMMANDS);   // check for the keyword

    // added echoing. Lajos, Jan 25, 2005
    if(eol == TRUE) {
      InFile.echoline(TRUE);
      continue;   // got to next line if end of line
    } else InFile.echoline(FALSE);
 
    // find keyword code
    if((CommandCode = FindKeywordCode(word)) < 0) {
      // VALID COMMAND NOT FOUND
      
      fprintf(output, " **** command ignored");   // invalid command
      continue;                          // goto next line 

    }

    // VALID COMMAND FOUND

    phase = GetCommandPhase(CommandCode);

    if(phase != 0) {                // check phase
      if(phase < lphase) {

        // INCORRECT CONTROL CARD SEQUENCE      
        fprintf(output, "\n **** command sequence error");
        fprintf(output, "\n **** program aborted\n");
        return -1;
      }

      // if end of execs close recfile
      // if changing phase to 4 or end encoutnered before phase 4
      if((lphase < 4) && (phase == 4)) {
        RecFile.Close();
      }

      lphase = phase;
    }
    else {
      if((CommandCode == SNARK_COMMAND_END) && (lphase < 4)) {
        RecFile.Close();
      }
    }
      
    // INSERT OVERLAY CALL BASED ON LPHASE HERE
    // PASS I-LPHASE*5 TO THE OVERLAY TO CALL THE RIGHT FUNCTION
    // DELETE THE COMPUTED GO TO AND ITS ASSOCIATED CALLS

    switch(CommandCode) {

//    case SNARK_COMMAND_COMENT: // comment
//      break;

    case SNARK_COMMAND_MODE: // ESTABLISH CONSTRAINT MODE
      mode();
      break;
      
    case SNARK_COMMAND_TRACE: // CHANGE THE TRACE SWITCH
      settra();
      break;
      
    case SNARK_COMMAND_END:
      ///fprintf(output, "\n         %10i storage units needed", maxsp);
      //fprintf(output, "\n         %10i storage units needed\n", 0);	// Obsolete

      // free the dinamically allocated global array objects
      if(MAX_NUMBER_OF_OBJECTS>0) delete[] objects;
      
      fprintf(output, "\n");
      return 0;

    case SNARK_COMMAND_CREATE: // CREATE TEST PATTERN AND/OR PROJECTION DATA
      if(File11.Open(TRUE) != 0) {
        fprintf(output, "\n **** unable to open file11 for wrting");
        fprintf(output, "\n **** program aborted\n");
	      return -1;
      }
      creatr();
      File11.Close();
      break;

    case SNARK_COMMAND_PICTURE: // SET PICT CALLED FLAG
      File11.isOpen = FALSE;
      rdpict(); //  READ TEST PATTERN
      iptflg = TRUE;
      break;

    case SNARK_COMMAND_PROJECTION: // BE SURE THAT PICTURE CMD HAS BEEN ENCOUNTERED
      // IF NOT  ERROR MSG  AND STOP
      if(iptflg) {
        rdproj(); // READ PROJECTION DATA
          
        if (File11.isOpen) File11.Close();
        iprflg = TRUE;
      }
      else {            
        ///write (output,1060)
        fprintf(output, "\n **** command picture is required before projection");
        fprintf(output, "\n **** program aborted\n");
        return -1;
      }
      break;

    case SNARK_COMMAND_BASSIS: // ESTABLISH THE BASIS 
      basis();   
      break;

    case SNARK_COMMAND_SELECT: // RAY SELECTION CONDITIONS FOR PICK
      select(FALSE);
       islflg = TRUE;                             //user specified select, by wei 1/05
      break;
      
    case SNARK_COMMAND_STOP: // ESTABLISH ALGORITHM TERMINATING CONDITIONS
      stopex();
      break;
      
    case SNARK_COMMAND_EXECUTE: // EXECUTE THE RECONSTRUCTION ALGORITHM
      if(iprflg) {
        if(!islflg) select(TRUE);        
        islflg = TRUE;          //no select command, select default efficient , by wei 1/05 
        exalg();
      }
      else {
        fprintf(output, "\n **** command projection is required before execute");
        fprintf(output, "\n **** program aborted\n");
        return -1;
      }
      break;
      
    case SNARK_COMMAND_EVALUATE: // EVALUATION OF THE RECONSTRUCTIONS
      Eval.eval_1();
      break;
      
    case SNARK_COMMAND_DISPLAY: // DIGITAL DISPLAY OF THE RECONSTRUCTIONS
      disply();
      break;
      
    case SNARK_COMMAND_PUNCH: // PUNCH THE RECONSTRUCTIONS
      pnch();
      break;
      
    case SNARK_COMMAND_LINES: // LINES DISPLAY OF THE DIFFERENCE
      doline();
      break;
      
    case SNARK_COMMAND_SKUNK: // PSEUDO HALF TONE DISPLSY OF THE RECONSTRUCTIONS
      halft();
      break;

    case SNARK_COMMAND_SUPERIORIZE: // ESTABLISH SUPERIORIZATION OPTIONS
      initSuperiorization();
      break;
    }
      
    second(&curr_time);
    dur = curr_time - begin;
    begin = curr_time;
    fprintf(output, "\n         %10.3f seconds used for processing command %s\n", dur, int2str(word)); // changed precision to three digits - swr 1/21/06
  };

  fprintf(output, "\n");
  return 0;
}

