/** @file lines.cpp
    @package snark14Display
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
        
    modification: Joanna Klukowska 1/13/2009
	    added choice of grayScale display for plot
	    in addition to color display
 */

#include "lines.hpp"
#include "displaylines.hpp"
#include "variables.hpp"

#include <stdio.h>
#include <qcombobox.h>
#include <qframe.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

/* "Lines" (plotting of pixel values in a row or column) dialog for a single image (and maybe phantom)
 *  Constructs a lineswindow which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
lineswindow::lineswindow(QWidget* parent, const char* name, double min, double max, bool modal, bool projrec, WFlags fl)
: QDialog(parent, name, modal, fl) {

    if (!name)
        setName("lineswindow");
    resize(340, 550);
    setMinimumSize(QSize(340, 550));
    setMaximumSize(QSize(340, 550));
    QString s = name; // dummy (multipurpose) string
    s += " Profile";
    setCaption(s);

    isrec = projrec;

    minval = min;
    maxval = max;

    rowcolbox = new QListBox(this, "rowcolbox");
    rowcolbox->insertItem("rows"); // rows first. => 0 => false  [ x-axis == columns ] [sic]
    rowcolbox->insertItem("columns"); // columns later => 1 => true [ x-axis == rows ] [sic]
    rowcolbox->setGeometry(QRect(20, 10, 80, 50));
    rowcolbox->setSelectionMode(QListBox::Single);
    rowcolbox->setSelected(0, true);
    QObject::connect(rowcolbox, SIGNAL(highlighted(int)), this, SLOT(updateXAxis(int)));

    phantombutton = new QRadioButton(this, "phantombutton");
    phantombutton->setGeometry(QRect(150, 20, 120, 30));
    phantombutton->setMinimumSize(QSize(120, 30));
    phantombutton->setMaximumSize(QSize(120, 30));
    phantombutton->setText("PHANTOM");
    if (isrec)
        phantombutton->setChecked(true);
    else {
        phantombutton->setChecked(false);
        phantombutton->setEnabled(false);
    }

    line1 = new QLineEdit(this, "line1");
    line1->setGeometry(QRect(20, 90, 60, 30));

    line2 = new QLineEdit(this, "line2");
    line2->setGeometry(QRect(100, 90, 60, 30));

    line3 = new QLineEdit(this, "line3");
    line3->setGeometry(QRect(180, 90, 60, 30));

    line4 = new QLineEdit(this, "line4");
    line4->setGeometry(QRect(260, 90, 60, 30));

    label1 = new QLabel(this, "label1");
    label1->setGeometry(QRect(33, 70, 35, 20));
    label1->setText("row1");

    label2 = new QLabel(this, "label2");
    label2->setGeometry(QRect(113, 70, 35, 20));
    label2->setText("row ");

    label3 = new QLabel(this, "label3");
    label3->setGeometry(QRect(193, 70, 35, 20));
    label3->setText("row3");

    label4 = new QLabel(this, "label4");
    label4->setGeometry(QRect(273, 70, 35, 20));
    label4->setText("row4");

    title = new QLineEdit(this, "title");
    title->setGeometry(QRect(20, 150, 300, 30));
    title->setText(name);

    titlelabel = new QLabel(this, "titlelabel");
    titlelabel->setGeometry(QRect(160, 130, 60, 20));
    titlelabel->setText("Title");

    //    subtitle =  new QComboBox( this, "subtitle" );
    //    subtitle->setGeometry( QRect( 20, 210, 140, 30 ) );
    //    subtitle->insertItem("Column/Line #",-1);
    //    subtitle->insertItem("Empty",-1);

    //    subtitlelabel = new QLabel( this, "subtitlelabel" );
    //    subtitlelabel->setGeometry( QRect( 60, 190, 100, 20 ) );
    //    subtitlelabel->setText( "Subtitle" );

    //    gridstyle = new QComboBox(this,"gridstyle");
    //    gridstyle->setGeometry( QRect( 180, 210, 140, 30 ) );
    //    gridstyle->insertItem("Solid",-1);
    //    gridstyle->insertItem("None",-1);
    //    gridstyle->insertItem("Dash",-1);
    //    gridstyle->insertItem("Dot",-1);
    //    gridstyle->setEnabled(false);
    //    gridstyle->setSelected(1); // "None"

    //    gridstylelabel = new QLabel( this, "gridstylelabel" );
    //    gridstylelabel->setGeometry( QRect( 210, 190, 210, 20 ) );
    //    gridstylelabel->setText( "Grid Style" );

    xaxis = new QLineEdit(this, "xaxis");
    xaxis->setGeometry(QRect(20, 240, 140, 30));
    xaxis->setText("Columns");

    yaxis = new QLineEdit(this, "yaxis");
    yaxis->setGeometry(QRect(180, 240, 140, 30));
    yaxis->setText("Pixel Values");

    xaxislabel = new QLabel(this, "xaxislabel");
    xaxislabel->setGeometry(QRect(70, 220, 60, 20));
    xaxislabel->setText("X Axis");

    yaxislabel = new QLabel(this, "yaxislabel");
    yaxislabel->setGeometry(QRect(230, 220, 60, 20));
    yaxislabel->setText("Y Axis");

    minx = new QLineEdit(this, "minx");
    minx->setGeometry(QRect(20, 300, 140, 30));
    minx->setText("0");

    s.setNum(isrec ? (Sizex - 1) : (usrays - 1)); // at the bottom, it's set to usrays... why?
    maxx = new QLineEdit(this, "maxx");
    maxx->setGeometry(QRect(180, 300, 140, 30));
    maxx->setText(s);

    s.setNum(minval);
    miny = new QLineEdit(this, "miny");
    miny->setGeometry(QRect(20, 360, 140, 30));
    miny->setText(s);

    s.setNum(maxval);
    maxy = new QLineEdit(this, "maxy");
    maxy->setGeometry(QRect(180, 360, 140, 30));
    maxy->setText(s);

    // these should be sort of exactly the same as in linesimages
    minxlabel = new QLabel(this, "minxlabel");
    minxlabel->setGeometry(QRect(60, 280, 60, 20));
    minxlabel->setText("Min Col");

    maxxlabel = new QLabel(this, "maxxlabel");
    maxxlabel->setGeometry(QRect(220, 280, 60, 20));
    maxxlabel->setText("Max Col");

    minylabel = new QLabel(this, "minylabel");
    minylabel->setGeometry(QRect(70, 340, 60, 20));
    minylabel->setText("Min Y");

    maxylabel = new QLabel(this, "maxylabel");
    maxylabel->setGeometry(QRect(230, 340, 60, 20));
    maxylabel->setText("Max Y");



    // jk 1/13/2009 adding grayScale option for graphs
    // jklukowska 6/7/2009 modified to be radio button with option of either
    // color or grayscale
    colorbuttongroup = new QButtonGroup(this, "colorbuttongroup");
    colorbuttongroup->setTitle(tr("Draw graphs in "));
    colorbuttongroup->setGeometry(QRect(0, 400, 340, 75));
    colorbuttongroup->setAlignment(int( QButtonGroup::AlignHCenter));
    grayScaleButton = new QRadioButton(colorbuttongroup, "grayScaleButton");
    grayScaleButton->setGeometry(QRect(30, 25, 130, 20));
    grayScaleButton->setText(tr(" grayscale"));
    grayScaleButton->setEnabled(true);
    grayScaleButton->setChecked(false);
    colorButton = new QRadioButton(colorbuttongroup, "colorButton");
    colorButton->setGeometry(QRect(190, 25, 130, 20));
    colorButton->setText(tr(" color"));
    colorButton->setEnabled(true);
    colorButton->setChecked(true);

    //Line3 = new QFrame(this, "Line3");
    //Line3->setGeometry(QRect(0, 485, 340, 16));
    //Line3->setFrameStyle(QFrame::HLine | QFrame::Sunken);

    showbutton = new QPushButton(this, "showbutton");
    showbutton->setGeometry(QRect(20, 490, 100, 40));
    showbutton->setText("Show");
    QObject::connect(showbutton, SIGNAL(clicked()), this, SLOT(accept()));

    cancelbutton = new QPushButton(this, "cancelbutton");
    cancelbutton->setGeometry(QRect(220, 490, 100, 40));
    cancelbutton->setText("Cancel");
    QObject::connect(cancelbutton, SIGNAL(clicked()), this, SLOT(reject()));

    updateXAxis(0); // another HACKish workaround by deniz for bug 98
}

/*  
 *  Destroys the object and frees any allocated resources
 */
lineswindow::~lineswindow() {
    // no need to delete child widgets, Qt does it all for us
}

/** 
@param void
@author Bruno M. Carvalho, a bit cleanup and change by deniz
@version 1.0 */
void lineswindow::updateXAxis(int i) {
    QString maxx_str;
    if (i == 0) { // (i==0), selection is Row, X Axis is columns
        label1->setText("row1");
        label2->setText("row2");
        label3->setText("row3");
        label4->setText("row4");
        xaxis->setText("Columns");
        minxlabel->setText("Min Col");
        maxxlabel->setText("Max Col");
        maxx_str.setNum(isrec ? (Sizex - 1) : (prjnum - 1));
        // i think "-1" for everything makes more sense--deniz
    } else { // (i==1), selection is Column, X Axis is rows
        label1->setText("col1");
        label2->setText("col2");
        label3->setText("col3");
        label4->setText("col4");
        xaxis->setText("Rows");
        minxlabel->setText("Min Row");
        maxxlabel->setText("Max Row");
        maxx_str.setNum(isrec ? (Sizey - 1) : (usrays - 1));
    }
    maxx->setText(maxx_str);
} // --lineswindow::updateXAxis()

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */
int lineswindow::getLinetype() {
    return rowcolbox->currentItem();
}

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */
int lineswindow::getLine1() {
    int l;
    bool ok;
    l = line1->text().toInt(&ok, 10);
    if (ok)
        return l;
    else
        return NaI;
}

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */
int lineswindow::getLine2() {
    int l;
    bool ok;
    l = line2->text().toInt(&ok, 10);
    if (ok)
        return l;
    else
        return NaI;
}

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */
int lineswindow::getLine3() {
    int l;
    bool ok;
    l = line3->text().toInt(&ok, 10);
    if (ok)
        return l;
    else
        return NaI;
}

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */
int lineswindow::getLine4() {
    int l;
    bool ok;
    l = line4->text().toInt(&ok, 10);
    if (ok)
        return l;
    else
        return NaI;
}

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */
QString lineswindow::getTitle() {
    return title->text();
}

//  /** 
//  @param void
//  @author Bruno M. Carvalho
//  @version 1.0 */ 
//  int lineswindow::getSubtitle()
//  {
//    return subtitle->currentItem();
//  }

//  /** 
//  @param void
//  @author Bruno M. Carvalho
//  @version 1.0 */ 
//  int lineswindow::getGridstyle()
//  {
//    return gridstyle->currentItem();
//  }

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */
QString lineswindow::getXAxis() {
    return xaxis->text();
}

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */
QString lineswindow::getYAxis() {
    return yaxis->text();
}

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */
int lineswindow::getMinX() {
    bool ok;
    int t;
    t = minx->text().toInt(&ok, 10);
    if (ok)
        return t;
    else
        return 0;
}

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */
int lineswindow::getMaxX() {
    bool ok;
    int t;
    t = maxx->text().toInt(&ok, 10);
    if (ok)
        return t;
    else
        return Sizex - 1;
}

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */
double lineswindow::getMinY() {
    bool ok;
    double t;
    t = miny->text().toDouble(&ok);
    if (ok)
        return t;
    else
        return minval;
}

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */
double lineswindow::getMaxY() {
    bool ok;
    double t;
    t = maxy->text().toDouble(&ok);
    if (ok)
        return t;
    else
        return maxval;
}

/** 
@param void
@author Bruno M. Carvalho
@version 1.0 */
bool lineswindow::isPhantomChecked() {
    return (phantombutton->isChecked());
}

/**
@param void
@author Joanna Klukowska (added 1/13/2009)
@version 1.0 */
bool lineswindow::isGrayScaleChecked() {
    return (grayScaleButton->isChecked());
}
