/** @file selectimages.cpp
    @package snark14Display
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

#include "selectimages.hpp"
#include "variables.hpp"

#include <malloc.h>
#include <qfont.h>
#include <qframe.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qmessagebox.h>

/* 
 *  Constructs a selectimageswindow which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
selectimageswindow::selectimageswindow( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    int i;

    if ( !name )
	setName( "selectimageswindow" );
    resize( 540, 350 ); 
    setMinimumSize( QSize( 540, 350 ) );
    setMaximumSize( QSize( 540, 350 ) );
    setCaption( tr( "Select Images" ) );

    okbutton = new QPushButton( this, "okbutton" );
    okbutton->setGeometry( QRect( 10, 15, 140, 40 ) ); 
    okbutton->setText( tr( "Ok" ) );
    QObject::connect(okbutton,SIGNAL(clicked()),this,SLOT(accept()));

    selectbutton = new QPushButton( this, "selectbutton" );
    selectbutton->setGeometry( QRect( 10, 65, 140, 40 ) ); 
    selectbutton->setText( tr( "Select All" ) );
    QObject::connect(selectbutton,SIGNAL(clicked()),this,SLOT(selectAll()));

    clearselectbutton = new QPushButton( this, "clearselectbutton" );
    clearselectbutton->setGeometry( QRect( 10, 115, 140, 40 ) ); 
    clearselectbutton->setText( tr( "Clear Selected" ) );
    QObject::connect(clearselectbutton,SIGNAL(clicked()),this,SLOT(clearSelected()));

    imageslistbox = new QListBox( this, "imageslistbox" );
    imageslistbox->setSelectionMode(QListBox::Multi);
    QFont f = QFont("courier",10,25,false);
    f.setFixedPitch(true);
    imageslistbox->setFont(f);
    for(i=0;i<numimages;i++)
      imageslistbox->insertItem(imagetitles[i]);
    imageslistbox->setGeometry( QRect( 160, 15, 370, 250 ) ); 

    Line6 = new QFrame( this, "Line6" );
    Line6->setGeometry( QRect( 0, 280, 540, 16 ) ); 
    Line6->setFrameStyle( QFrame::HLine | QFrame::Sunken );

    cancelbutton = new QPushButton( this, "cancelbutton" );
    cancelbutton->setGeometry( QRect( 300, 300, 100, 40 ) ); 
    cancelbutton->setText( tr( "Cancel" ) );
    QObject::connect(cancelbutton,SIGNAL(clicked()),this,SLOT(reject()));
}

/*  
 *  Destroys the object and frees any allocated resources
 */
selectimageswindow::~selectimageswindow()
{
    // no need to delete child widgets, Qt does it all for us
}

int* selectimageswindow::getSelectedImages()
{
  int i,j,nsi=0,*siarray;

  for(i=0;i<numimages;i++) 
    if(imageslistbox->isSelected(i))
       nsi++;

  siarray = (int*) malloc((unsigned) (nsi+1)*sizeof(int));
  if(!siarray) {
    QMessageBox::information(this,"SnarkDisplay","Error!\n"
                             "Allocation failure in getSelectedImages()\n");
  }
  else {
    siarray[0]=nsi;
    j=1;
    for(i=0;i<numimages;i++) 
      if(imageslistbox->isSelected(i)) {
	siarray[j]=i;
	j++;
      }
  }
  return siarray;
}

void selectimageswindow::selectAll()
{
  imageslistbox->selectAll(true);
}

void selectimageswindow::clearSelected()
{
  imageslistbox->selectAll(false);
}
