/** @file evaluate.cpp
    @package snark14Input
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

#include "evaluate.h"
#include "misc.h"
#include "variables.h"

#include <qbuttongroup.h>
#include <qframe.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qmultilineedit.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qmessagebox.h>

/** 
 *  Constructs a evaluatewindow which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
@param void
@author Bruno M. Carvalho
@version 1.0 */
evaluatewindow::evaluatewindow( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "evaluatewindow" );
    resize( 400, 600 ); 
    setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, sizePolicy().hasHeightForWidth() ) );
    setMinimumSize( QSize( 400, 600 ) );
    setMaximumSize( QSize( 400, 600 ) );
    setCaption( tr( "Evaluate Window" ) );

    evaluatelabel = new QLabel( this, "evaluatelabel" );
    evaluatelabel->setGeometry( QRect( 10, 10, 120, 30 ) ); 
    evaluatelabel->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, evaluatelabel->sizePolicy().hasHeightForWidth() ) );
    evaluatelabel->setMinimumSize( QSize( 120, 30 ) );
    evaluatelabel->setMaximumSize( QSize( 120, 30 ) );
    evaluatelabel->setText( tr( "EVALUATE" ) );

    evalnamelabel = new QLabel( this, "evalnamelabel" );
    evalnamelabel->setGeometry( QRect( 200, 20, 190, 30) );
    //    evalnamelabel->setMinimumSize( QSize( 120, 30) );
    evalnamelabel->setText( tr( "EVALUATE run name:" ) );

    evalnamefield = new QLineEdit( this, "evalnamefield" );
    evalnamefield->setGeometry( QRect( 210, 51, 170, 28) );
    evalnamefield->setEnabled(TRUE);

    evaluatebuttongroup = new QButtonGroup( this, "evaluatebuttongroup" );
    evaluatebuttongroup->setGeometry( QRect( 40, 40, 160, 140 ) ); 
    evaluatebuttongroup->setTitle( tr( "type" ) );
    evaluatebuttongroup->setAlignment( int( QButtonGroup::AlignHCenter ) );

    resolutionbutton = new QRadioButton( evaluatebuttongroup, "resolutionbutton" );
    resolutionbutton->setGeometry( QRect( 10, 20, 130, 24 ) ); 
    resolutionbutton->setText( tr( "RESOLUTION" ) );

    pointbutton = new QRadioButton( evaluatebuttongroup, "pointbutton" );
    pointbutton->setGeometry( QRect( 10, 50, 110, 24 ) ); 
    pointbutton->setText( tr( "POINT" ) );

    bothbutton = new QRadioButton( evaluatebuttongroup, "bothbutton" );
    bothbutton->setGeometry( QRect( 10, 80, 110, 24 ) ); 
    bothbutton->setText( tr( "BOTH" ) );

    nonebutton = new QRadioButton( evaluatebuttongroup, "nonebutton" );
    nonebutton->setGeometry( QRect( 10, 110, 80, 24 ) ); 
    nonebutton->setText( tr( "NONE" ) );

    linebutton = new QRadioButton( this, "linebutton" );
    //    linebutton->setGeometry( QRect( 210, 50, 100, 30 ) ); 
    linebutton->setGeometry( QRect( 210, 130, 100, 30 ) ); 
    linebutton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, linebutton->sizePolicy().hasHeightForWidth() ) );
    linebutton->setMinimumSize( QSize( 100, 30 ) );
    linebutton->setMaximumSize( QSize( 100, 30 ) );
    linebutton->setText( tr( "LINE" ) );

    evaluatebuttongroup2 = new QButtonGroup( this, "evaluatebuttongroup2" );
    evaluatebuttongroup2->setGeometry( QRect( 40, 190, 310, 80 ) ); 
    evaluatebuttongroup2->setTitle( tr( "" ) );
    evaluatebuttongroup2->setAlignment( int( QButtonGroup::AlignHCenter ) );

    wholepicbutton = new QRadioButton( evaluatebuttongroup2, "wholepicbutton" );
    wholepicbutton->setGeometry( QRect( 10, 10, 110, 24 ) ); 
    wholepicbutton->setText( tr( "WHOLEPIC" ) );
    QObject::connect(wholepicbutton,SIGNAL(clicked()),this,SLOT(activateWholepicFields()));
    wholepicbutton->setChecked(true);

    lowden = new QLineEdit( evaluatebuttongroup2, "lowden" );
    lowden->setGeometry( QRect( 150, 30, 60, 28 ) ); 
    lowden->setEnabled(TRUE);

    lowdenlabel = new QLabel( evaluatebuttongroup2, "lowdenlabel" );
    lowdenlabel->setGeometry( QRect( 150, 10, 60, 20 ) ); 
    lowdenlabel->setText( tr( "lowden" ) );
    lowdenlabel->setEnabled(TRUE);

    highden = new QLineEdit( evaluatebuttongroup2, "highden" );
    highden->setGeometry( QRect( 230, 30, 60, 28 ) );
    highden->setEnabled(TRUE);

    highdenlabel = new QLabel( evaluatebuttongroup2, "highdenlabel" );
    highdenlabel->setGeometry( QRect( 230, 10, 60, 20 ) ); 
    highdenlabel->setText( tr( "highden" ) );
    highdenlabel->setEnabled(TRUE);

    selectivebutton = new QRadioButton( evaluatebuttongroup2, "selectivebutton" );
    selectivebutton->setGeometry( QRect( 10, 40, 130, 24 ) ); 
    selectivebutton->setText( tr( "SELECTIVE" ) );
    QObject::connect(selectivebutton,SIGNAL(clicked()),this,SLOT(activateSelectiveFields()));

    shapes = new QMultiLineEdit( this, "shapes" );
    shapes->setGeometry( QRect( 30, 300, 350, 124 ) ); 
    shapes->setMaxLines(40);
    shapes->setEnabled(FALSE);

    shapeslabel = new QLabel( this, "shapeslabel" );
    shapeslabel->setGeometry( QRect( 90, 280, 240, 20 ) ); 
    shapeslabel->setText( tr( "SELECTIVE shapes descriptions" ) );
    shapeslabel->setEnabled(FALSE);

    shapesinstructionslabel = new QLabel( this, "shapesinstructionslabel" );
    shapesinstructionslabel->setGeometry( QRect( 80, 425, 270, 20 ) ); 
    shapesinstructionslabel->setText( tr( "shape cx cy u v ang [lowden higden]" ) );
    shapesinstructionslabel->setEnabled(FALSE);

    iterationflaglabel = new QLabel( this, "iterationflaglinelabel" );
    iterationflaglabel->setGeometry( QRect( 140, 450, 150, 20 ) ); 
    iterationflaglabel->setText( tr( "iteration_flag_line" ) );

    iterationflagline = new QLineEdit( this, "iterationflagline" );
    iterationflagline->setGeometry( QRect( 30, 470, 350, 28 ) ); 

    Line4 = new QFrame( this, "Line4" );
    Line4->setGeometry( QRect( -5, 505, 410, 16 ) ); 
    Line4->setFrameStyle( QFrame::HLine | QFrame::Sunken );

    okbutton = new QPushButton( this, "okbutton" );
    okbutton->setGeometry( QRect( 50, 540, 100, 40 ) ); 
    okbutton->setText( tr( "OK" ) );
    QObject::connect(okbutton,SIGNAL(clicked()),this,SLOT(checkValues()));

    cancelbutton = new QPushButton( this, "cancelbutton" );
    cancelbutton->setGeometry( QRect( 250, 540, 100, 40 ) ); 
    cancelbutton->setText( tr( "Cancel" ) );
    QObject::connect(cancelbutton,SIGNAL(clicked()),this,SLOT(reject()));
}

/**  
 *  Destroys the object and frees any allocated resources
 */
evaluatewindow::~evaluatewindow()
{
    // no need to delete child widgets, Qt does it all for us
}

/** Activates type, region and flag fields 
@param void
@author Bruno M. Carvalho
@version 1.0 */
/*void evaluatewindow::activateFields()
{
  if(evaluatebutton->isChecked()) {
    evaluatebuttongroup->setEnabled(TRUE);
    resolutionbutton->setEnabled(TRUE);
    pointbutton->setEnabled(TRUE);
    bothbutton->setEnabled(TRUE);
    nonebutton->setEnabled(TRUE);
    linebutton->setEnabled(TRUE);
    evaluatebuttongroup2->setEnabled(TRUE);
    wholepicbutton->setEnabled(TRUE);
    selectivebutton->setEnabled(TRUE);
    if(wholepicbutton->isChecked()) {
      lowden->setEnabled(TRUE);
      lowdenlabel->setEnabled(TRUE);
      highden->setEnabled(TRUE);
      highdenlabel->setEnabled(TRUE);
    }
    else 
      if(selectivebutton->isChecked()) {
        shapes->setEnabled(TRUE);
        shapeslabel->setEnabled(TRUE);
        shapesinstructionslabel->setEnabled(TRUE);
      }
    iterationflagline->setEnabled(TRUE);
    iterationflaglabel->setEnabled(TRUE);
  }
  else {
    evaluatebuttongroup->setEnabled(FALSE);
    resolutionbutton->setEnabled(FALSE);
    pointbutton->setEnabled(FALSE);
    bothbutton->setEnabled(FALSE);
    nonebutton->setEnabled(FALSE);
    linebutton->setEnabled(FALSE);
    evaluatebuttongroup2->setEnabled(FALSE);
    wholepicbutton->setEnabled(FALSE);
    selectivebutton->setEnabled(FALSE);
    lowden->setEnabled(FALSE);
    lowdenlabel->setEnabled(FALSE);
    highden->setEnabled(FALSE);
    highdenlabel->setEnabled(FALSE);
    shapes->setEnabled(FALSE);
    shapeslabel->setEnabled(FALSE);
    shapesinstructionslabel->setEnabled(FALSE);
    iterationflagline->setEnabled(FALSE);
    iterationflaglabel->setEnabled(FALSE);
  }
} */

/** Activates wholepic fields and deactivates selective fields
@param void
@author Bruno M. Carvalho
@version 1.0 */
void evaluatewindow::activateWholepicFields()
{
  if(wholepicbutton->isChecked()) {
    lowden->setEnabled(TRUE);
    lowdenlabel->setEnabled(TRUE);
    highden->setEnabled(TRUE);
    highdenlabel->setEnabled(TRUE); 
    shapes->setEnabled(FALSE);
    shapeslabel->setEnabled(FALSE);
    shapesinstructionslabel->setEnabled(FALSE);
  }
  else {
    lowden->setEnabled(FALSE);
    lowdenlabel->setEnabled(FALSE);
    highden->setEnabled(FALSE);
    highdenlabel->setEnabled(FALSE);
  }
}

/** Activates selective fields and deactivates wholepic fields 
@param void
@author Bruno M. Carvalho
@version 1.0 */
void evaluatewindow::activateSelectiveFields()
{
  if(selectivebutton->isChecked()) {
    shapes->setEnabled(TRUE);
    shapeslabel->setEnabled(TRUE);
    shapesinstructionslabel->setEnabled(TRUE);
    lowden->setEnabled(FALSE);
    lowdenlabel->setEnabled(FALSE);
    highden->setEnabled(FALSE);
    highdenlabel->setEnabled(FALSE);
  }
  else {
    shapes->setEnabled(FALSE);
    shapeslabel->setEnabled(FALSE);
    shapesinstructionslabel->setEnabled(FALSE);
  }
}

/** Returns a string for EVALUATE input sequence
@param void
@return evaluate
@author Bruno M. Carvalho
@version 1.0 */
QString evaluatewindow::getOutput()
{
  QString output;

  output.sprintf("EVALUATE ");
  if(resolutionbutton->isChecked()) {
    output.append("RESOLUTION ");
  }
  else
    if(pointbutton->isChecked()) {
      output.append("POINT ");
    }
    else
      if(bothbutton->isChecked()) {
        output.append("BOTH ");
      }
  if(linebutton->isChecked()) {
    output.append("LINE ");
  }

  output.append("\n");
  output += evalnamefield->text();
  output.append("\n");

  if(wholepicbutton->isChecked()) {
      output.append("WHOLEPIC ");
      if(! (lowden->text().stripWhiteSpace().isEmpty())) {
	output+=lowden->text();
	output.append(" ");
	output+=highden->text();
	output.append(" ");
      }
      output.append("\n");
  }
  else 
    if(selectivebutton->isChecked()) {
      output.append("SELECTIVE \n");
      output+=shapes->text();
      output.append(" \n");
      output.append("LAST \n");
    }
  output+=iterationflagline->text();
  //output.append(" \n");

  return output;
}


void evaluatewindow::checkValues()
{
  int f=0;
  float lo=0,hi=0;
  
  if(wholepicbutton->isChecked()) {
    bool lowblank  = lowden->text().stripWhiteSpace().isEmpty();
    bool highblank = highden->text().stripWhiteSpace().isEmpty();
    if((lowblank && (!highblank) )   ||   ( (!lowblank) && highblank)) {
      QMessageBox::information(this,"SnarkInput","Error!\n"
			       "Must specify either both or neither of lowden and highden");
      return;
    }
    if(!lowblank) {
      lo=strtof((const char *)lowden->text());
      if(errnum==1) 
	f=1;
      hi=strtof((const char *)highden->text());
      if(errnum==1) 
	f=2;
      if(lo>hi && !f) 
	f=3;
    }
  }
  if(!isintspace(iterationflagline->text()))
    f=4;

  QString teststr = evalnamefield->text();
  QString teststr2 = teststr.stripWhiteSpace();
  if (teststr2.isEmpty()) {
    f=5;
  }
  switch(f) {
  case 0: accept();
    break;
  case 1: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "Invalid lowden format");
          lowden->setFocus();
    break;
  case 2: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "Invalid highden format");
          highden->setFocus();
    break;
  case 3: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "lowden bigger than highden");
          lowden->setFocus();
    break;
  case 4: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "iteration_flag_line must contain\n"
                                   "only 0-9 and spaces and must\n"
                                   "be non-empty");
          iterationflagline->setFocus();
    break;
  case 5: QMessageBox::information(this,"SnarkInput","Error!\n"
				   "EVALUATE run name may NOT be\n"
				   "empty or whitespace");

    break;
  }
}

