/** @file select.cpp
    @package snark14Input
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

#include "select.h"
#include "variables.h"
#include "misc.h"
#include <stdio.h>

#include <qbuttongroup.h>
#include <qcombobox.h>
#include <qframe.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qmessagebox.h>

/** 
 *  Constructs a selectwindow which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
@param void
@author Bruno M. Carvalho
@version 1.0 */
selectwindow::selectwindow( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "selectwindow" );
    resize( 400, 350 ); 
    setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, sizePolicy().hasHeightForWidth() ) );
    setMinimumSize( QSize( 390, 350 ) );
    setMaximumSize( QSize( 390, 350 ) );
    setCaption( tr( "Select Window" ) );

    selectlabel = new QLabel( this, "selectlabel" );
    selectlabel->setGeometry( QRect( 10, 10, 90, 30 ) ); 
    selectlabel->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, selectlabel->sizePolicy().hasHeightForWidth() ) );
    selectlabel->setMinimumSize( QSize( 90, 30 ) );
    selectlabel->setMaximumSize( QSize( 90, 30 ) );
    selectlabel->setText( tr( "SELECT" ) );

    selectbuttongroup = new QButtonGroup( this, "selectbuttongroup" );
    selectbuttongroup->setGeometry( QRect( 20, 40, 110, 90 ) ); 
    selectbuttongroup->setTitle( tr( "type" ) );
    selectbuttongroup->setAlignment( int( QButtonGroup::AlignHCenter ) );

    userbutton = new QRadioButton( selectbuttongroup, "userbutton" );
    userbutton->setGeometry( QRect( 10, 20, 80, 24 ) ); 
    userbutton->setText( tr( "USER" ) ); 

    snarkbutton = new QRadioButton( selectbuttongroup, "snarkbutton" );
    snarkbutton->setGeometry( QRect( 10, 50, 80, 24 ) ); 
    snarkbutton->setText( tr( "SNARK" ) ); 
    snarkbutton->setChecked(true);

    //    QObject::connect(okbutton,SIGNAL(clicked()),this,SLOT(checkValues()));

    type1 = new QComboBox( this, "type1" );
    type1->setGeometry( QRect( 20, 170, 120, 28 ) ); 
    type1->insertItem("RAYSEQ",-1);
    type1->insertItem("PROJSEQ",-1);
    type1->insertItem("RANDOM",-1);

    type1label = new QLabel( this, "type1label" );
    type1label->setGeometry( QRect( 50, 150, 40, 20 ) ); 
    type1label->setText( tr( "type1" ) ); 

    n1 = new QLineEdit( this, "n1" );
    n1->setGeometry( QRect( 150, 170, 40, 28 ) ); 

    n1label = new QLabel( this, "n1label" );
    n1label->setGeometry( QRect( 160, 150, 20, 20 ) ); 
    n1label->setText( tr( "n1" ) ); 

    type2 = new QComboBox( this, "type2" );
    type2->setGeometry( QRect( 200, 170, 120, 28 ) );  
    type2->insertItem("",-1);
    type2->insertItem("RAYSEQ",-1);
    type2->insertItem("PROJSEQ",-1);
    type2->insertItem("RANDOM",-1);

    type2label = new QLabel( this, "type2label" );
    type2label->setGeometry( QRect( 230, 150, 40, 20 ) ); 
    type2label->setText( tr( "type2" ) ); 

    n2 = new QLineEdit( this, "n2" );
    n2->setGeometry( QRect( 330, 170, 40, 28 ) );

    n2label = new QLabel( this, "n2label" );
    n2label->setGeometry( QRect( 340, 150, 20, 20 ) ); 
    n2label->setText( tr( "n2" ) ); 

    steplabel = new QLabel( this, "steplabel" );
    steplabel->setGeometry( QRect( 20, 245, 50, 20 ) ); 
    steplabel->setText( tr( "STEP" ) ); 

    mod1 = new QLineEdit( this, "mod1" );
    mod1->setGeometry( QRect( 80, 240, 40, 28 ) ); 

    mod1label = new QLabel( this, "mod1label" );
    mod1label->setGeometry( QRect( 80, 220, 40, 20 ) ); 
    mod1label->setText( tr( "mod1" ) ); 

    mod2 = new QLineEdit( this, "mod2" );
    mod2->setGeometry( QRect( 130, 240, 40, 28 ) );

    mod2label = new QLabel( this, "mod2label" );
    mod2label->setGeometry( QRect( 130, 220, 40, 20 ) ); 
    mod2label->setText( tr( "mod2" ) ); 

    Line12 = new QFrame( this, "Line12" );
    Line12->setGeometry( QRect( 0, 275, 405, 16 ) ); 
    Line12->setFrameStyle( QFrame::HLine | QFrame::Sunken );

    efficientcb = new QCheckBox( this, "efficientcb" );
    efficientcb->setGeometry( QRect( 147, 90, 100, 24 ) );
    efficientcb->setText( "EFFICIENT" );
    efficientcb->setChecked(true);
    QObject::connect(efficientcb,SIGNAL(clicked()),this,SLOT(grayOrNot()));
    grayOrNot();

    okbutton = new QPushButton( this, "okbutton" );
    okbutton->setGeometry( QRect( 50, 290, 100, 40 ) ); 
    okbutton->setText( tr( "OK" ) );
    QObject::connect(okbutton,SIGNAL(clicked()),this,SLOT(checkValues()));

    cancelbutton = new QPushButton( this, "cancelbutton" );
    cancelbutton->setGeometry( QRect( 250, 290, 100, 40 ) ); 
    cancelbutton->setText( tr( "Cancel" ) );
    QObject::connect(cancelbutton,SIGNAL(clicked()),this,SLOT(reject()));

}

/** @author deniz
    @description grays out or ungrays out fields based on EFFICIENT
*/
void selectwindow::grayOrNot() {
  bool enab = (! (efficientcb->isChecked()));

  type1->setEnabled(enab);
  type1label->setEnabled(enab);
  type2->setEnabled(enab);
  type2label->setEnabled(enab);
  n1->setEnabled(enab);
  n2->setEnabled(enab);
  n1label->setEnabled(enab);
  n2label->setEnabled(enab);
  steplabel->setEnabled(enab);
  mod1->setEnabled(enab);
  mod2->setEnabled(enab);
  mod1label->setEnabled(enab);
  mod2label->setEnabled(enab);
}

/**  
 *  Destroys the object and frees any allocated resources
 */
selectwindow::~selectwindow()
{
    // no need to delete child widgets, Qt does it all for us
}

/** Returns a string for SELECT input sequence
    pre: assume checkValues() accepts
@param void
@return select
@author Bruno M. Carvalho [simplified / fixed bug #118 by Deniz]
@version 1.0 */
QString selectwindow::getOutput()
{
  QString output = "SELECT ";
  if(userbutton->isChecked()) {
    output += userbutton->text() + " ";
  } else {
    if(snarkbutton->isChecked()) {
      output += snarkbutton->text() + " ";
    }
  }
  if(efficientcb->isChecked()) {
    output += efficientcb->text();
  } else {
    output += type1->currentText() + " ";
    output += n1->text().simplifyWhiteSpace() + " ";
    if(type2->currentItem()) {
      output += type2->currentText() + " ";
      output += n2->text().simplifyWhiteSpace() + " ";
    }
    //  if passed checkValues(), add the next line iff mod1 is non-whitespace
    if (! (mod1->text().simplifyWhiteSpace().isEmpty())) {
      output += "\nSTEP " + 
	mod1->text().simplifyWhiteSpace() + " " +
	mod2->text().simplifyWhiteSpace();
    }
  }
  return output;
} // --selectwindow::getOutput()

/** @author deniz -- totally rewrote it while fixing bug #118
*/
void selectwindow::checkValues()
{

  if(efficientcb->isChecked()) { accept(); return; } // nothing can go wrong
  // otherwise...

  /**  the grammar of SELECT looks like:
    >  SELECT {USER, SNARK} [type1 [n1 type2 [n2] ] ]
      (where of course args in [] are optional and nesting them has the usual meaning)
      (this means that n1 is there iff type2 is there)
      cond, that is, ((type1==RANDOM) && (type2 does not appear)) XOR
      this line is followed by a line:
    >  STEP mod1 mod2
    (all textbox arguments must be positive integers)
    The new behavior coded by deniz:
      selectwindow forces user to choose exactly one of USER and SNARK, so ignore those.
      last 6 values: first check for existence and if exist, parse.
      if exist, report parse errors first depending on desired types ranges etc
      THEN check grammar (i.e., are "existences" compatible) and report grammar errors
  */
  // existence vars:
  bool n1E = false, type2E = false, n2E = false, mod1E = false, mod2E = false;
  bool type1isRandom = false, type2isRandom = false;
  int n1i, n2i, mod1i, mod2i; // integers to read values into for range checking

  type1isRandom = (type1->currentText().contains("RANDOM") ); // do NOT rely on position in combobox
  
  QString n1simple = n1->text().simplifyWhiteSpace();
  // strs in textboxes: first check if empty, then check if type and rangewise valid
  if (!n1simple.isEmpty()) {
    n1E = true;
    bool ok = true;
    n1i = n1simple.toInt(&ok);
    if(!ok) {
      QMessageBox::information(this,"SnarkInput","Error!\n"
			       "Invalid n1 format (must be positive integer).");
      n1->setFocus();
      return;
    } else if(n1i < 1) {
      QMessageBox::information(this,"SnarkInput","Error!\n"
			       "n1 must be greater than 0.");
      n1->setFocus();
      return;
    }
  }
  
  if(type2->currentItem()) { // if something is selected at all in combobox for type 2...
    type2E = true;
    type2isRandom = (type2->currentText().contains("RANDOM") ); // do NOT rely on position in combobox    
  }

  QString n2simple = n2->text().simplifyWhiteSpace();
  if(!n2simple.isEmpty()) {
    n2E = true;
    bool ok = true;
    n2i = n2simple.toInt(&ok);
    if(!ok) {
      QMessageBox::information(this,"SnarkInput","Error!\n"
			       "Invalid n2 format (must be positive integer).");
      n2->setFocus();
      return;
    } else if(n2i < 1) {
      QMessageBox::information(this,"SnarkInput","Error!\n"
			       "n2 must be greater than 0.");
      n2->setFocus();
      return;
    }
  }
  
  QString mod1simple = mod1->text().simplifyWhiteSpace();
  if(!mod1simple.isEmpty()) {
    mod1E = true;
    bool ok = true;
    mod1i = mod1simple.toInt(&ok);
    if(!ok) {
      QMessageBox::information(this,"SnarkInput","Error!\n"
			       "Invalid mod1 format (must be positive integer).");
      mod1->setFocus();
      return;
    } else if(mod1i < 1) {
      QMessageBox::information(this,"SnarkInput","Error!\n"
			       "mod1 must be greater than 0.");
      mod1->setFocus();
      return;
    }
  }
  
  QString mod2simple = mod2->text().simplifyWhiteSpace();
  if(!mod2simple.isEmpty()) {
    mod2E = true;
    bool ok = true;
    mod2i = mod2simple.toInt(&ok);
    if(!ok) {
      QMessageBox::information(this,"SnarkInput","Error!\n"
			       "Invalid mod2 format (must be positive integer).");
      mod2->setFocus();
      return;
    } else if(mod2i < 1) {
      QMessageBox::information(this,"SnarkInput","Error!\n"
			       "mod2 must be greater than 0.");
      mod2->setFocus();
      return;
    }
  }

  // check the grammar for "first line", from the outside in
  // if control reaches here, EFFICIENT not selected and type1 necessarily there
  if(  
     ( n2E && (!n1E))     || 
     ( type2E != n1E )
     ) {
    QMessageBox::information(this,"SnarkInput","Error!\n"
			     "Check your grammar--optional parameter dependencies are like:\n"
			     "[type1 [n1 type2 [n2] ] ]");
    return;
  }

  if(type2E) { // which of course implies type1E since we got here
    if(type1isRandom == type2isRandom) {
      QMessageBox::information(this,"SnarkInput","Error!\n"
			       "Since both type1 and type2 are present,\n"
			       "exactly one of them must be RANDOM");
      return;
    }
  }

  //  bool cond = (type1isRandom && (!type2E));
  if(! (type1isRandom && (!type2E)) )  { // "Unless cond, 'STEP' line must follow"
    if (! (mod1E && mod2E)) {
      QMessageBox::information(this,"SnarkInput","Error!\n"
			       "Check your grammar: "
			       "since it's not the case that type1 is random and type2 blank, \n"
			       "values must be entered for both mod1 and mod2");
      mod1->setFocus();
      return;
    }
  } else { // "Restrictions: 'STEP' must be absent if cond "
    if (mod1E || mod2E) {
      QMessageBox::information(this,"SnarkInput","Error!\n"
			       "Check your grammar: "
			       "since type1 is random and type2 is blank, \n"
			       "both mod1 and mod2 must be blank");
      mod1->setFocus();
      return;
    }
  }

  // control getting here implies the arguments were valid
  accept();    

} // --selectwindow::checkValues()
