/** @file stop.cpp
 @package snark14Input
 @author Bruno M. Carvalho and Deniz Sarioz
 licensed under (open-source) QPL v1.0
 which accompanies this distribution in the file QPL
 */

#include "stop.h"
#include "variables.h"
#include "misc.h"
#include <stdio.h>

#include <qbuttongroup.h>
#include <qframe.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qmessagebox.h>

/** 
 *  Constructs a stopwindow which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 @param void
 @author Bruno M. Carvalho
 @version 1.0 */
stopwindow::stopwindow(QWidget* parent, const char* name, bool modal, WFlags fl) :
		QDialog(parent, name, modal, fl)
{
	if (!name)
		setName("stopwindow");
	resize(600, 340);
	setSizePolicy(
			QSizePolicy((QSizePolicy::SizeType) 0, (QSizePolicy::SizeType) 0,
					sizePolicy().hasHeightForWidth()));
	setMinimumSize(QSize(600, 340));
	setMaximumSize(QSize(600, 340));
	setCaption(tr("Stop Window"));

	stoplabel = new QLabel(this, "stoplabel");
	stoplabel->setGeometry(QRect(10, 10, 70, 30));
	stoplabel->setSizePolicy(
			QSizePolicy((QSizePolicy::SizeType) 0, (QSizePolicy::SizeType) 0,
					stoplabel->sizePolicy().hasHeightForWidth()));
	stoplabel->setMinimumSize(QSize(70, 30));
	stoplabel->setMaximumSize(QSize(70, 30));
	stoplabel->setText(tr("STOP"));

	stopbuttongroup = new QButtonGroup(this, "stopbuttongroup");
	stopbuttongroup->setGeometry(QRect(15, 45, 575, 205));
	stopbuttongroup->setTitle(tr("type"));
	stopbuttongroup->setAlignment(int(QButtonGroup::AlignHCenter));

	iterationbutton = new QRadioButton(stopbuttongroup, "iterationbutton");
	iterationbutton->setGeometry(QRect(10, 45, 118, 24));
	iterationbutton->setText(tr("ITERATION"));
	QObject::connect(iterationbutton, SIGNAL(clicked()), this,
			SLOT(activateIterationFields()));

	terminationbutton = new QRadioButton(stopbuttongroup, "terminationbutton");
	terminationbutton->setGeometry(QRect(10, 100, 135, 24));
	terminationbutton->setText(tr("TERMINATION"));
	QObject::connect(terminationbutton, SIGNAL(clicked()), this,
			SLOT(activateTerminationFields()));

	nonebutton = new QRadioButton(stopbuttongroup, "nonebutton");
	nonebutton->setGeometry(QRect(10, 150, 75, 24));
	nonebutton->setText(tr("NONE"));
	QObject::connect(nonebutton, SIGNAL(clicked()), this,
			SLOT(deactivateFields()));

	iter = new QLineEdit(stopbuttongroup, "iter");
	iter->setGeometry(QRect(150, 40, 80, 30));
	iter->setSizePolicy(
			QSizePolicy((QSizePolicy::SizeType) 0, (QSizePolicy::SizeType) 0,
					iter->sizePolicy().hasHeightForWidth()));
	iter->setEnabled(FALSE);

	iterlabel = new QLabel(stopbuttongroup, "iterlabel");
	iterlabel->setGeometry(QRect(180, 20, 40, 20));
	iterlabel->setText(tr("iter"));
	iterlabel->setEnabled(FALSE);

	testname = new QComboBox(stopbuttongroup, "testname");
	testname->setGeometry(QRect(150, 100, 130, 30));
	testname->setSizePolicy(
			QSizePolicy((QSizePolicy::SizeType) 0, (QSizePolicy::SizeType) 0,
					testname->sizePolicy().hasHeightForWidth()));
	testname->insertItem("VARIANCE", -1);
	testname->insertItem("MLST", -1);
	testname->insertItem("KLDS", -1);
	testname->insertItem("RESI", -1);
	testname->insertItem("WSQD", -1);
	testname->insertItem("TRM1", -1);
	testname->insertItem("TRM2", -1);
	testname->setEnabled(FALSE);
	QObject::connect(testname,SIGNAL(activated(int)),this,SLOT(setEpsilon()));

	testnamelabel = new QLabel(stopbuttongroup, "testnamelabel");
	testnamelabel->setGeometry(QRect(175, 80, 80, 20));
	testnamelabel->setText(tr("test_name"));
	testnamelabel->setEnabled(FALSE);

	epsilon = new QLineEdit(stopbuttongroup, "epsilon");
	epsilon->setGeometry(QRect(290, 100, 80, 30));
	epsilon->setSizePolicy(
			QSizePolicy((QSizePolicy::SizeType) 0, (QSizePolicy::SizeType) 0,
					epsilon->sizePolicy().hasHeightForWidth()));
	epsilon->setEnabled(FALSE);

	epsilonlabel = new QLabel(stopbuttongroup, "epsilonlabel");
	epsilonlabel->setGeometry(QRect(305, 80, 60, 20));
	epsilonlabel->setText(tr("epsilon"));
	epsilonlabel->setEnabled(FALSE);

	rprtcb = new QCheckBox( this, "rprtcb" );
	rprtcb->setGeometry(QRect(400, 145, 80, 30));
	rprtcb->setText( "RPRT" );
	rprtcb->setChecked(false);
	rprtcb->setEnabled(FALSE);
	QObject::connect(rprtcb,SIGNAL(clicked()),this,SLOT(setSkips()));

	skips = new QLineEdit(stopbuttongroup, "skips");
	skips->setGeometry(QRect(470, 100, 80, 30));
	skips->setSizePolicy(
			QSizePolicy((QSizePolicy::SizeType) 0, (QSizePolicy::SizeType) 0,
					skips->sizePolicy().hasHeightForWidth()));
	skips->setEnabled(FALSE);

	skipslabel = new QLabel(stopbuttongroup, "skipslabel");
	skipslabel->setGeometry(QRect(485, 80, 60, 20));
	skipslabel->setText(tr("skips"));
	skipslabel->setEnabled(FALSE);

	Line17 = new QFrame(this, "Line17");
	Line17->setGeometry(QRect(-5, 265, 415, 16));
	Line17->setFrameStyle(QFrame::HLine | QFrame::Sunken);

	okbutton = new QPushButton(this, "okbutton");
	okbutton->setGeometry(QRect(50, 290, 100, 40));
	okbutton->setText(tr("OK"));
	QObject::connect(okbutton, SIGNAL(clicked()), this, SLOT(checkValues()));

	cancelbutton = new QPushButton(this, "cancelbutton");
	cancelbutton->setGeometry(QRect(280, 290, 100, 40));
	cancelbutton->setText(tr("Cancel"));
	QObject::connect(cancelbutton, SIGNAL(clicked()), this, SLOT(reject()));
}

/**  
 *  Destroys the object and frees any allocated resources
 */
stopwindow::~stopwindow()
{
	// no need to delete child widgets, Qt does it all for us
}

/** Activates iteration fields 
 @param void
 @author Bruno M. Carvalho
 @version 1.0 */
void stopwindow::activateIterationFields()
{
	if (iterationbutton->isChecked())
	{
		iter->setEnabled(TRUE);
		iterlabel->setEnabled(TRUE);
		testname->setEnabled(FALSE);
		testnamelabel->setEnabled(FALSE);
		epsilon->setEnabled(FALSE);
		epsilonlabel->setEnabled(FALSE);
	}
	else
	{
		iter->setEnabled(FALSE);
		iterlabel->setEnabled(FALSE);
	}
}

/** Activates termination fields 
 @param void
 @author Bruno M. Carvalho
 @version 1.0 */
void stopwindow::activateTerminationFields()
{
	if (terminationbutton->isChecked())
	{
		testname->setEnabled(TRUE);
		testnamelabel->setEnabled(TRUE);
		iter->setEnabled(FALSE);
		iterlabel->setEnabled(FALSE);
		setEpsilon();
	}
	else
	{
		testname->setEnabled(FALSE);
		epsilon->setEnabled(FALSE);
		epsilonlabel->setEnabled(FALSE);
		testnamelabel->setEnabled(FALSE);
	}
}

/** Deactivates iteration and termination subfields 
 @param void
 @author Bruno M. Carvalho
 @version 1.0 */
void stopwindow::deactivateFields()
{
	if (nonebutton->isChecked())
	{
		iter->setEnabled(FALSE);
		iterlabel->setEnabled(FALSE);
		testname->setEnabled(FALSE);
		epsilon->setEnabled(FALSE);
		epsilonlabel->setEnabled(FALSE);
		testnamelabel->setEnabled(FALSE);
	}
	else
	{
		iter->setEnabled(TRUE);
		iterlabel->setEnabled(TRUE);
		testname->setEnabled(TRUE);
		testnamelabel->setEnabled(TRUE);
		setEpsilon();
	}
}

void stopwindow::setEpsilon()
{
	if (testname->currentText().contains("VARIANCE") || testname->currentText().contains("KLDS") || testname->currentText().contains("RESI") || testname->currentText().contains("WSQD"))
	{
		epsilon->setEnabled(TRUE);
		epsilonlabel->setEnabled(TRUE);
	}
	else
	{
		epsilon->setEnabled(FALSE);
		epsilonlabel->setEnabled(FALSE);
	}

	if (testname->currentText().contains("MLST") || testname->currentText().contains("KLDS") || testname->currentText().contains("RESI") || testname->currentText().contains("WSQD"))
	{
		rprtcb->setEnabled(TRUE);
	}
	else
	{
		rprtcb->setEnabled(FALSE);
	}
}

void stopwindow::setSkips()
{
	if (rprtcb->isChecked())
	{
		skips->setEnabled(TRUE);
		skipslabel->setEnabled(TRUE);
	}
	else
	{
		skips->setEnabled(FALSE);
		skipslabel->setEnabled(FALSE);
	}
}

/** Returns a string for STOP input sequence
 @param void
 @return stop
 @author Bruno M. Carvalho
 @version 1.0 */
QString stopwindow::getOutput()
{
	QString output;

	output.sprintf("STOP ");
	if (iterationbutton->isChecked())
	{
		output.append("ITERATION ");
		output += iter->text();
		output.append(" ");
	}
	if (terminationbutton->isChecked())
	{
		output.append("TERMINATION ");
		switch (testname->currentItem())
		{
		case 0:
			output.append("VARIANCE");
			break;
		case 1:
			output.append("MLST");
			break;
		case 2:
			output.append("KLDS");
			break;
		case 3:
			output.append("RESI");
			break;
		case 4:
			output.append("WSQD");
			break;
		case 5:
			output.append("TRM1");
			break;
		case 6:
			output.append("TRM2");
			break;
		}
		output.append(" ");
		if (testname->currentText().contains("VARIANCE") || testname->currentText().contains("KLDS") || testname->currentText().contains("RESI") || testname->currentText().contains("WSQD"))
		{
			output += epsilon->text();
		}

		if (rprtcb->isChecked())
		{
			output.append(" RPRT");

			float s = strtof((const char *) skips->text());
			if (s>1)
			{
				output.append(" ");
				output += skips->text();
			}
		}
	}

	return output;
}

void stopwindow::checkValues()
{
	int f = 0, i = 0;
	float e = 0;
	char str1[5];

	str1[4] = 0;
	if (iterationbutton->isChecked())
	{
		i = strtoi((const char *) iter->text());
		if (errnum)
			f = 1;
		if (i < 1)
			f = 2;
	}
	if (terminationbutton->isChecked())
	{
		if (testname->currentText().contains("VARIANCE") || testname->currentText().contains("KLDS") || testname->currentText().contains("RESI") || testname->currentText().contains("WSQD"))
		{
			e = strtof((const char *) epsilon->text());
			if (errnum)
				f = 3;
			if (e <= 0)
				f = 4;
		}
	}

	switch (f)
	{
	case 0:
		accept();
		break;
	case 1:
		QMessageBox::information(this, "SnarkInput", "Error!\n"
				"Invalid iter format.");
		iter->setFocus();
		break;
	case 2:
		QMessageBox::information(this, "SnarkInput", "Error!\n"
				"iter must be greater than 0.");
		iter->setFocus();
		break;
	case 3:
		QMessageBox::information(this, "SnarkInput", "Error!\n"
				"Invalid epsilon format.");
		epsilon->setFocus();
		break;
	case 4:
		QMessageBox::information(this, "SnarkInput", "Error!\n"
				"epsilon must be greater than 0.");
		epsilon->setFocus();
		break;
	}
}
